# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Log event parser capability.

Parser provides general purpose regular expression matching on a line by line
basis with the ability to extract both the last and history of matching events.

When the Parser class is created it loads filter files or filter file
directories specified in the `filters` argument.

JSON filter file format (version 1.0):
* A single JSON object with the following keys:

    * "description" whose value is a string (optional)
    * "version" whose value is a JSON object with the following keys:

        * "major" whose value is an integer that specifies the major version of
        the file
        * "minor" whose value is an integer that specifies the major version of
        the file

    * "filters" whose value is an array of Filter JSON objects with the
    following keys:

        * "description" whose value is a string (optional)
        * "regex_match" whose value is a regular expression string
        * "name" whose value is a string

::

    {
        "description": "helpful optional description for all filters contained",
        "version": {"major": 1, "minor": 0},
        "filters": [
            {
                "description": "optional description for this specific filter",
                "regex_match": "required regular expression string",
                "name": "required_name"
            }
        ]
    }

As each JSON filter file is loaded and parsed each regular expression is
assigned a name consisting of the JSON filter file name (without the extension)
plus the name assigned to each Filter object (the "name" key) separated with a
'.' (e.g. errors.assert). This is known in the Parser code base as the
"event_label" and is used as a lookup key to retrieve Parser filter events.

The Parser class expects to store all matching events in an event file. To
reduce the memory footprint of the Parser class all matching events are
immediately written to the event file by the process_line method. The caller
must provide the shared dynamic event dictionary, an event file object, and the
raw log line to this method. Keeping the shared dynamic event dictionary and the
opening and closing of the event file outside the Parser class helps to simplify
the Parser class logic and ensures that the file is closed by the caller as
needed (e.g. exception handling).

For each regular expression that matches the raw log line provided to the
process_line method the shared dynamic event dictionary is updated. When all
regular expressions have been processed an event JSON object containing the
results of all matches by "event_lable" is written (and flushed) to the event
file. The event JSON object format is shown as follows:

JSON event file format (version 1.0):
* A single JSON object with the following keys:

    * "raw_log_line" whose value is the raw log line string parsed
    * "system_timestamp" whose value is a string containing the timestamp
    portion of the
      raw log line
    * "event_label" for each regular expression that matched the log line and
    whose value is an
      array of regular expression group data extracted

::

    {
        "system_timestamp": "2018-01-30 12:05:59.118746",
        "state.shutdown_reboot": ["The system is shutting down"],
        "raw_log_line": "<2018-01-30 12:05:59.118746> announce: The system is
        shutting down."
    }

Since event history can only be obtained from the event file, all event history
commands require the path to the event file to retrieve this information. Event
history is obtained by using the "tac" and "grep" unix tools to filter matching
events in the event file and from Python extract and decode each event JSON
object as described above.
"""
import datetime
import json
import os
import re
import subprocess
import time
from typing import Collection

from gazoo_device import decorators
from gazoo_device import errors
from gazoo_device import gdm_logger
from gazoo_device.capabilities.interfaces import event_parser_base

logger = gdm_logger.get_logger("parser")

FILTER_JSON_FORMAT_MAJOR_VERSION = 1
TIMESTAMP_FORMAT = "%Y-%m-%d %H:%M:%S.%f"
ParserResult = event_parser_base.ParserResult


def _get_full_filter_name(name, filter_path):
  """Returns full filter name using information from name and filter_path provided.

  Args:
      name (str): as specified in filter dictionary in filter file
      filter_path (str): absolute path to filter file

  Returns:
      str: Full filter name to be used in Parser event file as event_label

  Note:
      The full filter name consists of the following elements:
          * filename without extension (e.g. crashes)
          * name as specified for each filter in filter file (e.g. segfault)

      Example:
          crashes.segfault
  """
  filename = os.path.basename(filter_path)
  file_only = os.path.splitext(filename)[0]
  filter_name = "{}.{}".format(file_only, name)
  return filter_name


def _get_datetime(dt_value):
  """Convert string to datetime object.

  Args:
      dt_value (str): string to convert.

  Raises:
      RuntimeError: if datetime string formatting fails to convert the value

  Returns:
      datetime: object converted from value

  Note:
    Looks for 1 or 3 formats:
       "%Y-%m-%d %H:%M:%S.%f %z"
       "%Y-%m-%d %H:%M:%S.%f"
       "%Y-%m-%d %H:%M:%S"
  """
  result = None
  if result is None:
    result = _get_datetime_from_format(dt_value, "%Y-%m-%d %H:%M:%S.%f %z")
  if result is None:
    result = _get_datetime_from_format(dt_value, "%Y-%m-%d %H:%M:%S.%f")
  if result is None:
    result = _get_datetime_from_format(dt_value, "%Y-%m-%d %H:%M:%S")
  if result is None:
    raise RuntimeError(
        "Failed to convert '{}' into datetime object".format(dt_value))
  return result


def _get_datetime_from_format(dt_value, dt_format):
  """Convert value provided into datetime object or None if not successful.

  The purpose of this method is to attempt to convert the value provided
  using the datetime format string also provided into a datetime object. If
  the value provided doesn't correspond to the datetime format provided
  then the datetime strptime method will throw a ValueError that this method
  will catch and consume by returning None for the result. This reduces the
  error checking the calling function must perform when testing multiple
  datetime formats that may or may not successfully convert the value
  provided into a datetime object.

  Args:
      dt_value (str): the datetime string value to be converted to a datetime
        object
      dt_format (str): the datetime format to use in conversion process

  Returns:
      datetime: if value can be formatted with dt_format provided into
      datetime object or None
      if unsuccessful.
  """
  result = None
  try:
    result = datetime.datetime.strptime(dt_value, dt_format)
  except ValueError:
    pass
  return result


def _get_last_event(device_event_file_path, event_label, timeout=1.0):
  """Returns event data matching the event label specified or None.

  Args:
      device_event_file_path (str): path to file with filtered event
        information.
      event_label (str): event label to lookup (e.g. 'x.state'). If None,
        return the last event regardless of event label.
      timeout (float): Timeout value in seconds. Example: 1.5.

  Raises:
      ValueError: if unable to read json data found in event_file_path.

  Returns:
      tuple: (dict, bool) where the first item is a dict which contains result
      data, and the second item is True if the command timed out, False
      otherwise. The result dict contains data in the following format::

          {"system_timestamp": datetime,
           "raw_log_line": str,
           event_label: list,
           ...}

      If event_label is provided, only the matching event label will be
      included;
      otherwise, all labels recorded in the event file will be included.
  """
  result = None
  timedout = False

  file_exists, remaining_timeout = _wait_for_event_file(device_event_file_path,
                                                        timeout)
  if not file_exists:
    timedout = True
    return result, timedout

  timeout_str = "{:f}".format(remaining_timeout)

  if event_label is None:
    tail_cmd = [
        "timeout", timeout_str, "tail", "-n", "1", device_event_file_path
    ]
    out = ""
    try:
      out = subprocess.check_output(tail_cmd).decode("utf-8", "replace")
    except subprocess.CalledProcessError as err:
      if err.returncode == 124:
        timedout = True
    events = _get_events_from_json_output([out], event_label)
  elif event_label:
    tac_cmd = ["tac", device_event_file_path]
    grep_cmd = ["timeout", timeout_str, "grep", "-m", "1", "-w", event_label]
    tac_proc = subprocess.Popen(tac_cmd, stdout=subprocess.PIPE)
    grep_proc = subprocess.Popen(
        grep_cmd, stdin=tac_proc.stdout, stdout=subprocess.PIPE)
    tac_proc.stdout.close()
    out, _ = grep_proc.communicate()
    if grep_proc.returncode == 124:
      timedout = True
    tac_proc.terminate()
    events = _get_events_from_json_output([out], [event_label])
  else:
    events = None

  if events:
    result = events[0]

  return result, timedout


def _get_events_from_json_output(json_events, event_labels=None):
  """Returns list of event data history matching event_labels provided.

  Args:
      json_events (list): of JSON object strings to return history for.
      event_labels (list): list of event labels to lookup (e.g. ['x.state',
        'x.restored'])

  Raises:
      ValueError: when parsing JSON objects from json_events fails

  Returns:
      list: a list of dictionary objects for each event with the following
      format:
          {"system_timestamp": timestamp as a datetime object,
           "raw_log_line": raw log line as a string
           event_label: a list of regex group data matches, empty list if no
           groups
           ...}

      If multiple event labels match the same line, there will be one
      event_label
      in the dictionary for each match.
  """
  result = []
  for json_object in json_events:
    if json_object:
      try:
        event_dict = json.loads(json_object)
      except ValueError as err:
        logger.info(
            "Failed to parse event log line; skipping. Err: {!r}".format(err))
        continue
      if event_labels is not None:
        filtered_dict = {
            "system_timestamp": _get_datetime(event_dict["system_timestamp"]),
            "raw_log_line": event_dict["raw_log_line"]
        }
        for event_label in event_labels:
          if event_label in event_dict:
            filtered_dict[event_label] = event_dict[event_label]
        result.append(filtered_dict)
      else:
        event_dict["system_timestamp"] = _get_datetime(
            event_dict["system_timestamp"])
        result.append(event_dict)
  return result


def _get_all_event_history(device_event_file_path, event_labels, timeout=10.0):
  """Returns list of event history matching event labels specified.

  Args:
      device_event_file_path (str): path to file with filtered event
        information
      event_labels (list): list of event labels to lookup (e.g. ['x.state',
        'x.restored']). If None, return all events.
      timeout (float): Timeout value in seconds. Example: 10.0.

  Raises:
      ValueError: on (unlikely) invalid JSON data found in
      self.event_file_path file.

  Returns:
     tuple: (list of event objects (dict), bool whether or not command
     timedout )

  Note:
      The Parser standard is to return the list from newest to oldest,
      therefore we reverse the output returned by grep.
  """
  result = []
  timedout = False

  file_exists, remaining_timeout = _wait_for_event_file(device_event_file_path,
                                                        timeout)
  if not file_exists:
    timedout = True
    return result, timedout

  timeout_str = "{:f}".format(remaining_timeout)

  if event_labels is None:
    tac_cmd = ["timeout", timeout_str, "tac", device_event_file_path]
    out = ""
    try:
      out = subprocess.check_output(tac_cmd).decode("utf-8", "replace")
    except subprocess.CalledProcessError as err:
      if err.returncode == 124:
        timedout = True
    json_events = out.splitlines()
  else:
    grep_cmd = ["timeout", timeout_str, "grep", "-w"]
    for event_label in event_labels:
      if event_label:
        grep_cmd.append("-e")
        grep_cmd.append(event_label)
    grep_cmd.append(device_event_file_path)
    grep_proc = subprocess.Popen(grep_cmd, stdout=subprocess.PIPE)
    out, _ = grep_proc.communicate()
    if grep_proc.returncode == 124:
      timedout = True
    json_events = out.splitlines()
    json_events.reverse()

  return _get_events_from_json_output(json_events, event_labels), timedout


def _get_event_history_count(device_event_file_path, event_label, timeout=10.0):
  """Returns count of event history matching event label specified.

  Args:
      device_event_file_path (str): to extract event history from
      event_label (str): event label to lookup (e.g. 'x.state')
      timeout (float): Timeout value in seconds. Example: 10.0.

  Raises:
      ValueError: on (unlikely) invalid JSON data found in
      self.event_file_path file.

  Returns:
      tuple: (int count of all events found, bool of whether command timed
      out).
  """
  result = 0
  timedout = False

  file_exists, remaining_timeout = _wait_for_event_file(device_event_file_path,
                                                        timeout)
  if not file_exists:
    timedout = True
    return result, timedout

  timeout_str = "{:f}".format(remaining_timeout)

  grep_cmd = [
      "timeout", timeout_str, "grep", "-c", "-w", event_label,
      device_event_file_path
  ]
  grep_proc = subprocess.Popen(grep_cmd, stdout=subprocess.PIPE)
  out, _ = grep_proc.communicate()
  if grep_proc.returncode == 124:
    timedout = True

  if out:
    result = int(out.strip())

  return result, timedout


def _get_limited_event_history(device_event_file_path,
                               event_labels,
                               limit,
                               timeout=10.0):
  """Returns list of event history matching event labels specified.

  Args:
      device_event_file_path(str): path to file with filtered event
        information
      event_labels(list): list of event labels to lookup(e.g. ['x.state',
        'x.restored']). If None, return all events up to limit.
      limit(int): the number of event data elements returned
      timeout(float): Timeout value in seconds. Example: 10.0.

  Raises:
      ValueError: on(unlikely) invalid JSON data found in self.event_file_path
      file

  Returns:
      tuple: (list of event dicts up to 'limit' length, bool whether or not
      timed out)

  Note:
      Event info listed in most recent to least recent order. If regex_match
      for event_label
      has groups, it will return the group match, else it will return the raw
      log line.
      Implemented via grep and tac for time efficiency.
  """
  result = []
  timedout = False

  file_exists, remaining_timeout = _wait_for_event_file(device_event_file_path,
                                                        timeout)
  if not file_exists:
    timedout = True
    return result, timedout

  timeout_str = "{:f}".format(remaining_timeout)

  if event_labels is None:
    tac_cmd = ["tac", device_event_file_path]
    head_cmd = ["timeout", timeout_str, "head", "-n", str(limit)]
    tac_proc = subprocess.Popen(tac_cmd, stdout=subprocess.PIPE)
    head_proc = subprocess.Popen(
        head_cmd, stdin=tac_proc.stdout, stdout=subprocess.PIPE)
    tac_proc.stdout.close()
    out, _ = head_proc.communicate()
    if head_proc.returncode == 124:
      timedout = True
    tac_proc.terminate()
    json_events = out.splitlines()
  else:
    tac_cmd = ["tac", device_event_file_path]
    grep_cmd = ["timeout", timeout_str, "grep", "-m", str(limit), "-w"]
    for event_label in event_labels:
      if event_label:
        grep_cmd.append("-e")
        grep_cmd.append(event_label)
    tac_proc = subprocess.Popen(tac_cmd, stdout=subprocess.PIPE)
    grep_proc = subprocess.Popen(
        grep_cmd, stdin=tac_proc.stdout, stdout=subprocess.PIPE)
    tac_proc.stdout.close()
    out, _ = grep_proc.communicate()
    if grep_proc.returncode == 124:
      timedout = True
    tac_proc.terminate()
    json_events = out.splitlines()

  return _get_events_from_json_output(json_events, event_labels), timedout


def _wait_for_event_file(event_file_path, timeout):
  """Wait up to the timeout period for the event file to be created.

  Args:
      event_file_path(str): Full path to the event file.
      timeout(float): Timeout value in seconds. Example: 1.5.

  Returns:
      tuple: (bool, float) A tuple where the first value is True if the file
      was found before the timeout expired, and False otherwise, and the second
      value is the amount of timeout time remaining.  For example, (True, 0.958)
      would indicate the file was found with 0.958 seconds of timeout left, and
      (False, 0) indicates that the event file was not found.  If the file is
      not found, the remaining time will always be 0.
  """

  start_time = time.time()
  end_time = start_time + timeout

  while True:
    timeout_remaining = end_time - time.time()
    if timeout_remaining <= 0:
      return (False, 0)
    elif os.path.exists(event_file_path):
      return (True, timeout_remaining)
    else:
      time.sleep(0.1)


class _EventMatch(object):
  """Encapsulates matching events with event time delta."""

  def __init__(self, event_delta, event_list):
    """Initializes _EventMatch.

    Args:
        event_delta (timedelta): Difference between event datetime instances
        event_list (list): Containing filter events

    Note:
        Filter events in event_list typically appear as::  [
            {"basic.reboot_trigger": [],
            "system_timestamp": datetime.datetime.strptime("2018-06-18
              15:13:48.011747",
                                                           "%Y-%m-%d
                                                             %H:%M:%S.%f"),
            "raw_log_line": "Note: GDM triggered reboot"},
            {"basic.bootup": [],
            "system_timestamp": datetime.datetime.strptime("2018-06-18
              15:13:49.092755",
                                                           "%Y-%m-%d
                                                             %H:%M:%S.%f"),
            "raw_log_line": "[0:00:00.035] [APPL] Welcome to Lock App"} ]
    """

    self.event_delta = event_delta
    self.event_list = event_list


class EventParserDefault(event_parser_base.EventParserBase):
  """Parser class for filtering log lines."""

  def __init__(self,
               filters: Collection[str],
               event_file_path: str,
               device_name: str) -> None:
    """Initializes the log event parser.

    Args:
        filters: Paths to JSON filter files or filter directories.
        event_file_path: Path to the log event file.
        device_name: The name of the device using this capability.
    """
    super().__init__(device_name=device_name)
    self._filters_dict = {}
    self.event_file_path = event_file_path
    self.load_filters(filters)

  def get_event_history(self, event_labels=None, count=None, timeout=10.0):
    r"""Returns up to count elements of event data matching given list of event labels.

    Args:
        event_labels (list): list of event labels to lookup(example:
          ["power.state", "power.lost"]). If None(default), return all
          events.
        count (int): of event data elements to return (Default: None for all
          events).
        timeout (float): Timeout value in seconds. Example: 10.0.

    Raises:
        ParserError: on error parsing event file.

    Returns:
        ParserResult: An object containing events matching any of the event
        labels provided in event_labels, or all available events if event_labels
        in None. The event data is stored in the .results_list attribute and the
        count of results in the .count attribute. If no matching events are
        found, the .results_list attribute will be set to the empty list.

        If the .timedout attribute is True, the operation timed out before all
        events were searched, or the event file was not found before the timeout
        expired.

    Notes:
        Event info is listed in most recent to least recent order.

        The following examples use a hypothetical power.json filter file
        with the following filters:

        .. code-block:: none

            {
                "name": "state",
                "regex_match": "power:(\\w+)"
            },
            {
                "name": "lost",
                "regex_match": "power lost"
            },
            {
                "name": "restored",
                "regex_match": "power restored"
            }

        The original log file being read looks like this:

        .. code-block:: none

            <2018-02-02 10:32:07.167234> [APPL] Spoke: power restored,
            powering on
            <2018-02-02 12:00:57.154328> [APPL] Spoke: power lost, powering
            down
            <2018-02-02 12:00:57.154328> power:OFF
            <2018-02-02 12:01:22.122824> [APPL] Spoke: power restored,
            powering on
            <2018-02-02 12:01:22.122824> power:ON
            <2018-02-02 12:02:18.121948> [APPL] Spoke: power lost, powering
            down
            <2018-02-02 12:02:18.121948> power:OF

        Example output with event_labels ["power.state"], showing group
        matches:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime.datetime(2018, 2, 2, 12, 2,
                18, 121948),
                  'raw_log_line': 'power:OFF',
                  'power.state': ['OFF']},
                 {'system_timestamp': datetime.datetime(2018, 2, 2, 12, 1,
                 22, 122824),
                  'raw_log_line': 'power:ON',
                  'power.state': ['ON']},
                 {'system_timestamp': datetime.datetime(2018, 2, 2, 12, 0,
                 57, 154328),
                  'raw_log_line': 'power:OFF',
                  'power.state': ['OFF']}]
            result.count = 3

        Example output with event_labels ["power.lost"], showing matches
        with no groups:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime.datetime(2018, 2, 2, 12, 2,
                18, 121948),
                  'raw_log_line: '[APPL] Spoke: power lost, powering down',
                  'power.lost': []},
                 {'system_timestamp': datetime.datetime(2018, 2, 2, 12, 0,
                 57, 154328),
                  'raw_log_line': '[APPL] Spoke: power lost, powering down',
                  'power.lost': []}]
            result.count = 2

        Example output with event_labels["power.restored"], showing matches
        with no groups and count=1:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime.datetime(2018, 2, 2, 12, 1,
                22, 122824),
                  'raw_log_line': '[APPL] Spoke: power restored, powering
                  on',
                  'power.restored': []}
            result.count = 1

        Example output with event_labels ["power.lost", "power.restored"]:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime.datetime(2018, 2, 2, 12, 2,
                18, 121948),
                  'raw_log_line: '[APPL] Spoke: power lost, powering down',
                  'power.lost': []},
                 {'system_timestamp': datetime.datetime(2018, 2, 2, 12, 1,
                 22, 122824),
                  'raw_log_line': '[APPL] Spoke: power restored, powering
                  on',
                  'power.restored': []},
                 {'system_timestamp': datetime.datetime(2018, 2, 2, 12, 0,
                 57, 154328),
                  'raw_log_line': '[APPL] Spoke: power lost, powering down',
                  'power.lost': []}
                 {'system_timestamp': datetime.datetime(2018, 2, 2, 10, 32,
                 7, 167234),
                  'raw_log_line': '[APPL] Spoke: power restored, powering
                  on',
                  'power.restored': []}]
            result.count = 4
    """
    if event_labels:
      self.verify_event_labels(
          event_labels,
          error_message="{} get_event_history failed.".format(
              self._device_name))
    timedout = False
    try:
      if count:
        history_results, timedout = _get_limited_event_history(
            self.event_file_path, event_labels, count, timeout=timeout)
      else:
        history_results, timedout = _get_all_event_history(
            self.event_file_path, event_labels, timeout=timeout)
      return ParserResult(
          timedout=timedout,
          results_list=history_results,
          count=len(history_results))
    except Exception as err:
      raise errors.ParserError(
          "Retrieving events {} history from {} failed. Error {!r}".format(
              event_labels, self.event_file_path, err))

  def get_event_history_count(self, event_label, timeout=10.0):
    """Returns count of event history matching event label specified.

    Args:
        event_label(str): event label to lookup(e.g. power.state)
        timeout(float): Timeout value in seconds. Example: 10.0.

    Raises:
        ParserError: on error parsing event file

    Returns:
        ParserResult: An object containing the count of results in the
        .count attribute. The .results_list field is set to the empty list.

        If the .timedout attribute is True, the operation timed out before
        all events were searched, or the event file was not found before the
        timeout expired.
    """

    self.verify_event_labels(
        [event_label],
        error_message="%s get_event_history_count failed." % self._device_name)

    try:
      count, timedout = _get_event_history_count(
          self.event_file_path, event_label, timeout=timeout)
      return ParserResult(timedout=timedout, results_list=[], count=count)
    except Exception as err:
      raise errors.ParserError(
          "Retrieving event {} history from {} failed. Error {!r}".format(
              event_label, self.event_file_path, err))

  def get_event_label_dict(self, pattern=None):
    """Takes an event label pattern, and returns a dictionary of matching event labels.

    Args:
        pattern(str): Regex for event labels. Default: None

    Returns:
        dict: containing keys(i.e. event labels), and values(i.e regular
        expressions)

    Raises:
        ParserError: if pattern provided is an invalid regular expression

    Note:
        The pattern regular expression provided will be matched strictly
        against the beginning of each event_label.
        If no pattern is provided, the _filters_dict is returned.
    """

    if pattern is None:
      return {key: value.pattern for key, value in self._filters_dict.items()}
    else:
      try:
        result_dict = {}
        regex = re.compile(pattern)
        for event_label in list(filter(regex.match, list(self._filters_dict))):
          result_dict[event_label] = self._filters_dict[event_label].pattern
        return result_dict
      except Exception as err:
        raise errors.ParserError(
            "Compiling regular expression pattern {} failed. "
            "Error: {!r}".format(pattern, err))

  def get_event_labels(self, pattern=None):
    """Takes an event label pattern, and returns a list of matching event labels.

    Args:
        pattern(str):  Regex for event labels. Default: None

    Returns:
        list: of matching event label strings

    Raises:
        ParserError: if pattern provided is an invalid regular expression

    Note:
        The pattern regular expression provided will be matched strictly
        against the beginning of each event_label.
        If no pattern is provided, all filter event labels are returned.
    """

    if pattern is None:
      return list(self._filters_dict.keys())
    else:
      try:
        regex = re.compile(pattern)
        return list(filter(regex.match, list(self._filters_dict.keys())))
      except Exception as err:
        raise errors.ParserError(
            "Compiling regular expression pattern {} failed. "
            "Error {!r}".format(pattern, err))

  def get_last_event(self, event_labels=None, timeout=1.0):
    r"""Returns the most recent matching event for each item in the list of event labels.

    Args:
        event_labels (list): list of event labels to lookup (example:
          ["power.state", "power.lost"]). None (default) returns the most
          recent event regardless of label.
        timeout (float): Timeout value in seconds. Example: 1.5.

    Raises:
        ParserError: on error parsing event file.
        DeviceError: Label doesn't exist or
                     event_labels is not type list

    Returns:
        ParserResult: An object containing the most recent event match for
        each event label in the event_labels list, or the most recent event if
        event_labels is None. The event data is stored in the .results_list
        attribute, and the count of results is in the .count attribute. If
        no matching events are found, the .results_list attribute will be set to
        the empty list.

        If the .timedout attribute is True, the operation timed out before
        all events were searched, or the event file was not found before the
        timeout expired.

    Notes:
        The event_label is the desired filter filename without extension a .
        and the filter name (e.g. power.state).

        The following examples use a hypothetical power.json filter file
        with the following filters:

        .. code-block:: none

            {
                "name": "state",
                "regex_match": "power:(\\w+)"
            },
            {
                "name": "lost",
                "regex_match": "power lost"
            },
            {
                "name": "restored",
                "regex_match": "power restored"
            }

        The original log file being read looks like this:

        .. code-block:: none

            <2018-02-02 10:32:07.167234> [APPL] Spoke: power restored,
            powering on
            <2018-02-02 12:00:57.154328> [APPL] Spoke: power lost, powering
            down
            <2018-02-02 12:00:57.154328> power:OFF
            <2018-02-02 12:01:22.122824> [APPL] Spoke: power restored,
            powering on
            <2018-02-02 12:01:22.122824> power:ON
            <2018-02-02 12:02:18.121948> [APPL] Spoke: power lost, powering
            down
            <2018-02-02 12:02:18.121948> power:OFF

        Example output with event_labels=["power.state"], showing group
        matches:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime.datetime(2018, 2, 2, 12, 2,
                18, 121948),
                  'raw_log_line': 'power:OFF',
                  'power.state': ['OFF']}]
            result.count = 1

        Example output with event_labels=["power.lost"], showing matches
        with no groups:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime.datetime(2018, 2, 2, 12, 2,
                18, 121948),
                  'raw_log_line': '[APPL] Spoke: power lost',
                  'power.lost': []}]
            result.count = 1

        Example output with event_labels=["power.state", "power.lost",
        "power.restored"]:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime(2018, 2, 2, 12, 2, 18,
                121948),
                  'raw_log_line': 'power:OFF',
                  'power.state': ['OFF']},
                 {'system_timestamp': datetime(2018, 2, 2, 12, 2, 18,
                 121948),
                  'raw_log_line': '[APPL] Spoke: power lost',
                  'power.lost': []}
                 {'system_timestamp': datetime(2018, 2, 2, 12, 1, 22,
                 122824),
                  'raw_log_line': '[APPL] Spoke: power restored, powering
                  on',
                  'power.restored': []}]
            results.count = 3
    """
    if event_labels is None:
      event_labels = [None]
    else:
      self.verify_event_labels(
          event_labels,
          error_message="%s get_last_event failed." % self._device_name)

    results = []
    any_timed_out = False

    try:
      for event_label in event_labels:
        event_data, timedout = _get_last_event(self.event_file_path,
                                               event_label, timeout)
        if event_data:
          results.append(event_data)
        any_timed_out |= timedout

      return ParserResult(
          timedout=any_timed_out, results_list=results, count=len(results))

    except Exception as err:
      raise errors.ParserError(
          "{} get_last_event failed for {}. Error: {!r}.".format(
              self._device_name, event_labels, err))

  def get_last_event_state(self,
                           event_label,
                           event_name=None,
                           raise_error=False,
                           timeout=1.0,
                           group_index=0):
    r"""Return the matching group of the event label regex if one is found.

    Args:
        event_label(str): The event label based on the filter name and
          filter file name.
        event_name (str): name of event. Defaults to event_label.
        raise_error(bool): Raise an exception if the event has not been seen
          or otherwise return an empty string.
        timeout (float): time to parse events.
        group_index (int): index of the match group to return (default 0 for
          first).

    Returns:
        str: The result from the call to get_last_event and getting the
        first group.

    Raises:
        ParserError: if there is an error getting the last event.

        .. code-block:: none
            {
                "name": "state",
                "regex_match": "power:(\\w+)"
            }

        The original log file being read looks like this:

        .. code-block:: none

            <2018-02-02 10:32:07.167234> [APPL] Spoke: power restored,
            powering on
            <2018-02-02 12:00:57.154328> [APPL] Spoke: power lost, powering
            down
            <2018-02-02 12:00:57.154328> power:OFF
            <2018-02-02 12:01:22.122824> [APPL] Spoke: power restored,
            powering on
            <2018-02-02 12:01:22.122824> power:ON
            <2018-02-02 12:02:18.121948> [APPL] Spoke: power lost, powering
            down
            <2018-02-02 12:02:18.121948> power:OFF

        With event_labels=["power.state"], it would return 'OFF'
    """
    if not event_name:
      event_name = event_label
    result = self.get_last_event([event_label], timeout=timeout)

    if not result.results_list:
      msg = "{} get event state failed. {} not yet seen in logs".format(
          self._device_name, event_name)
    else:
      match_groups = result.results_list[0][event_label]
      if group_index < len(match_groups):
        return match_groups[group_index]
      else:
        msg = (
            "{} get event state failed. Group index ({}) is greater than "
            "number of match groups ({})").format(self._device_name, group_index,
                                        match_groups)

    if raise_error:
      raise errors.ParserError(msg)
    else:
      logger.warning(msg)
      return ""

  def get_unexpected_reboots(self):
    """Device's event file is used to identify unmatched reboot events.

    Returns:
        list: unexpected reboot event

    Note:
        For example in the following case there are two reboot triggers but
        only one
        bootup event::

            {"basic.reboot_trigger": [],
             "system_timestamp": datetime.datetime.strptime("2018-06-18
             15:13:48.011747",
                                                            "%Y-%m-%d
                                                            %H:%M:%S.%f"),
             "raw_log_line": "Note: GDM triggered reboot"},
            {"basic.reboot_trigger": [],
             "system_timestamp": datetime.datetime.strptime("2018-06-18
             15:13:52.825231",
                                                            "%Y-%m-%d
                                                            %H:%M:%S.%f"),
             "raw_log_line": "Note: GDM triggered reboot"}
            {"basic.bootup": [],
             "system_timestamp": datetime.datetime.strptime("2018-06-18
             15:13:58.540417",
                                                            "%Y-%m-%d
                                                            %H:%M:%S.%f"),
             "raw_log_line": "[0:00:00.035] [APPL] Welcome to Lock App"}

        This method would return::

            [{"basic.reboot_trigger": [],
             "system_timestamp": datetime.datetime.strptime("2018-06-18
             15:13:48.011747",
                                                            "%Y-%m-%d
                                                            %H:%M:%S.%f"),
             "raw_log_line": "Note: GDM triggered reboot"}]
    """

    _, remaining_bootups = self.get_unmatched_events(
        event_cause_label="basic.reboot_trigger",
        event_effect_label="basic.bootup")

    return remaining_bootups

  def get_unmatched_events(self,
                           event_cause_label="basic.reboot_trigger",
                           event_effect_label="basic.bootup"):
    """Device's event file is used to identify unmatched events.

    Args:
        event_cause_label (str): Name of event label causing event
        event_effect_label (str): Name of event label signifying effect
          event

    Returns:
        tuple:  list of unmatched cause event dict entries
                list of unmatched effect event dict entries

    Note:
        For example in the following case there are two reboot triggers but
        only one bootup event::

            {"basic.reboot_trigger": [],
             "system_timestamp": datetime.datetime.strptime("2018-06-18
             15:13:48.011747",
                                                            "%Y-%m-%d
                                                            %H:%M:%S.%f"),
             "raw_log_line": "Note: GDM triggered reboot"},
            {"basic.reboot_trigger": [],
             "system_timestamp": datetime.datetime.strptime("2018-06-18
             15:13:52.825231",
                                                            "%Y-%m-%d
                                                            %H:%M:%S.%f"),
             "raw_log_line": "Note: GDM triggered reboot"}
            {"basic.bootup": [],
             "system_timestamp": datetime.datetime.strptime("2018-06-18
             15:13:58.540417",
                                                            "%Y-%m-%d
                                                            %H:%M:%S.%f"),
             "raw_log_line": "[0:00:00.035] [APPL] Welcome to Lock App"}

        This method would return the following tuple::

            ([{"basic.reboot_trigger": [],
             "system_timestamp": datetime.datetime.strptime("2018-06-18
             15:13:48.011747",
                                                            "%Y-%m-%d
                                                            %H:%M:%S.%f"),
             "raw_log_line": "Note: GDM triggered reboot"}],
            [])
    """
    event_cause_list = []  # Cause events matching event_cause_label
    event_match_list = [
    ]  # EventMatch objects holding events matched together based on time
    remaining_cause_list = []  # Cause events with no matching effect events
    remaining_effect_list = []  # Effect events with no matching cause events

    event_cause_hist = self.get_event_history([event_cause_label])
    if not event_cause_hist.timedout:
      remaining_cause_list = event_cause_hist.results_list[:]
      event_cause_list = event_cause_hist.results_list
    event_effect_hist = self.get_event_history([event_effect_label])
    if not event_effect_hist.timedout:
      remaining_effect_list = event_effect_hist.results_list[:]

    # List of each cause event's datetime.datetime value
    cause_datetime_list = [
        event["system_timestamp"] for event in event_cause_list
    ]

    # A copy of cause_datetime_list adjusted during event match processing
    wrk_datetime_list = cause_datetime_list[:]

    # Create list of matching events
    for effect_event in event_effect_hist.results_list:
      effect_datetime = effect_event["system_timestamp"]
      try:
        nearest_cause_datetime = self._nearest_datetime(wrk_datetime_list,
                                                        effect_datetime)
      except errors.ParserError:  # Means wrk_datetime_list is empty
        break
      delta_time = effect_datetime - nearest_cause_datetime
      cause_event = event_cause_list[cause_datetime_list.index(
          nearest_cause_datetime)]
      event_match = _EventMatch(
          event_delta=delta_time, event_list=[cause_event, effect_event])
      event_match_list.append(event_match)
      # The datetime has been used, adjust list to avoid reuse
      del wrk_datetime_list[wrk_datetime_list.index(nearest_cause_datetime)]
    if len(event_match_list) > 1:
      event_match_list.sort(key=lambda match_obj: match_obj.event_delta)

    # Use list of matching events to determine unmatched events
    for event_match in event_match_list:
      if remaining_cause_list and remaining_effect_list:
        cause_event = event_match.event_list[0]
        effect_event = event_match.event_list[1]
        try:
          idx = remaining_cause_list.index(cause_event)
          del remaining_cause_list[idx]
        except ValueError:
          pass
        try:
          idx = remaining_effect_list.index(effect_event)
          del remaining_effect_list[idx]
        except ValueError:
          pass
    return remaining_cause_list, remaining_effect_list

  @decorators.CapabilityLogDecorator(logger, level=decorators.DEBUG)
  def load_filter_file(self, filter_path):
    """Load JSON filter file specified.

    Args:
        filter_path (str): path to JSON filter file

    Raises:
        ParserError: when loading JSON filter file fails or on unexpected
        JSON key values

    Note:
        Only files ending in .json are loaded, others are skipped

        Example:
            {
                "description": "Assertion event filter",
                "version": {"major": 1, "minor": 0},
                "filters": [
                    {
                        "description": "match and extract assert message"
                        "regex_match": "ASSERT: (.+)",
                        "name": "state"
                    }
                ]
            }
        """
    logger.debug("Adding filter file %s", filter_path)
    try:
      with open(filter_path, "r") as filter_file:
        try:
          json_filter_data = json.load(filter_file)
        except Exception as err:
          msg = "Unable to parse filter file {} as a json file. {!r}".format(
              filter_path, err)
          logger.debug(msg)
          raise errors.ParserError(msg)
    except IOError:
      raise errors.ParserError(
          "Unable to access filter path '{}'".format(filter_path))

    if "version" not in json_filter_data:
      raise errors.ParserError(
          "Loading filter-file {} failed. Missing 'version' key.".format(
              filter_path))

    if "filters" not in json_filter_data:
      raise errors.ParserError(
          "Loading filter-file {} failed. Missing 'filters' key.".format(
              filter_path))

    if not isinstance(json_filter_data["version"], dict):
      raise errors.ParserError(
          "Loading filter-file {} failed. "
          "Expecting value of 'version' entry to be a dictionary "
          "but instead its a {}.".format(filter_path,
                                         type(json_filter_data["version"])))

    version_info = json_filter_data["version"]

    if "major" not in version_info:
      raise errors.ParserError(
          "Loading filter-file {} failed. "
          "Missing 'major' key in 'version' value.".format(filter_path))

    if "minor" not in version_info:
      raise errors.ParserError(
          "Loading filter-file {} failed. "
          "Missing 'minor' key in 'version' value.".format(filter_path))

    if not isinstance(version_info["major"], int):
      raise errors.ParserError(
          "Loading filter-file {} failed. "
          "Expecting int for major version found {} instead.".format(
              filter_path, type(version_info["major"])))

    if not isinstance(version_info["minor"], int):
      raise errors.ParserError(
          "Loading filter-file {} failed. "
          "Expecting int for minor version found {} instead.".format(
              filter_path, type(version_info["minor"])))

    if version_info["major"] != FILTER_JSON_FORMAT_MAJOR_VERSION:
      raise errors.ParserError(
          "Loading filter-file {} failed. "
          "Found unexpected major version in JSON filter file.".format(
              filter_path))

    self._add_filters(json_filter_data["filters"], filter_path)

  @decorators.CapabilityLogDecorator(logger, level=None)
  def process_line(self,
                   event_file,
                   raw_log_line,
                   header_length=29,
                   log_filename=""):
    """Parser line against filters and write JSON result to event_file provided.

    Args:
        event_file (file): to write matching filter results to
        raw_log_line (str): to match filters against
        header_length (int): added by GDM to strip off from raw_log_line
        log_filename (str): name of log file raw_log_line came from

    Note: The expected format for the raw_log_line is one that contains the
      system timestamp between characters 1 and 27 as shown in the following
      example:
        <2018-02-01 14:39:00.492376> ...
        A line containing a event_entry should be written as follows:
            * Key: log_filename, Value: filename where raw_log_line was found
            * Key: matched_timestamp, Value: a host timestamp when match was
              found.
            * Key: raw_log_line, Value: the line provided without any trailing
              whitespace
            * Key: system_timestamp, Value: the timestamp portion of the raw log
              line
            * Key: <event_label>, Value: regular expression group tuple as
              an array which might be empty  In addition to writing results
              to the event file the <event_label> is used as a key to update
              the shared dynamic events dictionary with the same information
              to allow for easier access to the data for the last matching
              <event_label> in the main process
    """

    event_data = {}
    for filter_name, regex in self._filters_dict.items():
      match = regex.search(raw_log_line)
      if match:
        match_data = match.groups()
        event_data[filter_name] = match_data

    if event_data:
      if log_filename:
        event_data["log_filename"] = log_filename
      event_data["raw_log_line"] = raw_log_line.rstrip()[header_length:]
      event_data["system_timestamp"] = raw_log_line[1:27]
      event_data["matched_timestamp"] = \
          datetime.datetime.now().strftime(TIMESTAMP_FORMAT)
      event_file.write(json.dumps(event_data) + "\n")
      event_file.flush()

  @decorators.CapabilityLogDecorator(logger, level=decorators.DEBUG)
  def verify_event_labels(self, event_labels, error_message=""):
    """Verifies event_labels in correct format and exist.

    Args:
       event_labels (list): list of event_labels.
       error_message (str): error_message to use as header.

    Raises:
       ParserError: if bad format or certain labels don't exist.
    """
    if event_labels is not None and not isinstance(event_labels, list):
      raise errors.ParserError("{} Expecting event_labels to be a list "
                               "or None but received {} instead.".format(
                                   error_message, type(event_labels)))
    if any(not isinstance(label, str) for label in event_labels):
      raise errors.ParserError(
          "{} Expected event_labels to be a list of strings. "
          "Found {}.".format(error_message, event_labels))

    cur_event_labels = self.get_event_labels()
    for event_label in event_labels:
      if event_label not in cur_event_labels:
        raise errors.ParserError(
            "{} label {} not available. Choose from: {}".format(
                self._device_name, event_label, self.get_event_labels()))

  def wait_for_event_labels(self,
                            event_labels,
                            raise_error=False,
                            timeout=20.0,
                            start_datetime=None):
    """Waits up to timeout seconds for event labels to appear in device logs.

    Args:
        event_labels (list): labels(strings) to wait for. Will wait for at
          least one of each.
        raise_error (bool): raise an error if the labels are not found in
          time.
        timeout (float): seconds to wait for labels to appear in logs.
        start_datetime (datetime): events before this time will be ignored.

    Raises:
        ParserError: if input format is bad
        ParserError: raise_error is True and not all labels were found.

    Returns:
        bool: whether all event labels were found.
    """
    self.verify_event_labels(
        event_labels,
        error_message="{} wait_for_event_labels failed.".format(
            self._device_name))
    logger.info("{} waiting up to {}s for the following labels: {}.".format(
        self._device_name, timeout, event_labels))

    if start_datetime is None:
      start_datetime = datetime.datetime.now()
    start_time = time.time()
    end_time = start_time + timeout

    all_found = False

    while not all_found and time.time() < end_time:
      remaining_time = timeout - (time.time() - start_time)
      # len(event_labels) can be 0; avoid zero division
      time_per_label = remaining_time / max(len(event_labels), 1)

      any_timed_out = False
      found_labels = []
      missed_labels = []

      for event_label in event_labels:
        result = self.get_last_event([event_label], timeout=time_per_label)
        any_timed_out |= result.timedout
        if (result.results_list and
            result.results_list[0]["system_timestamp"] >= start_datetime):
          found_labels.append(event_label)
        else:
          missed_labels.append(event_label)

      all_found = not any_timed_out and len(found_labels) == len(event_labels)

    if not all_found:
      msg = (
          "{}: not all events corresponding to labels were found in {}s. "
          "Looked for labels: {}. Found labels: {}. Missed labels: {}.".format(
              self._device_name, timeout, event_labels, found_labels,
              missed_labels))
      if raise_error:
        raise errors.ParserError(msg)
      else:
        logger.warning(msg)

    return all_found

  def _add_filter(self, filter_list, filter_path):
    """Adds filter dictionary expression from filter_path specified.

    Args:
        filter_list(list): of filter dictionaries definitions to add
        filter_path(str): that provided the list of filters being added

    Raises:
        ParserError: when one of the following occurs
            * filter dictionary is missing the name key
            * filter dictionary is missing the regex_match key
            * full filter name already exists(e.g. already loaded)
            * the regex_match is invalid(won't compile)
    """
    if "name" not in filter_list:
      raise errors.ParserError(
          "Loading filter-file {} failed. "
          "Missing 'name' key in filter object '{!r}'.".format(
              filter_path, filter_list))
    if "regex_match" not in filter_list:
      raise errors.ParserError(
          "Loading filter-file {} failed. "
          "Missing 'regex_match' key in filter object '{!r}'.".format(
              filter_path, filter_list))

    full_filter_name = _get_full_filter_name(filter_list["name"], filter_path)
    if full_filter_name in self._filters_dict:
      raise errors.ParserError("Loading filter-file {} failed. "
                               "Filter named {} already exists.".format(
                                   filter_path, full_filter_name))

    try:
      self._filters_dict[full_filter_name] = re.compile(
          filter_list["regex_match"])
      logger.debug("Added filter %s from filter file %s", full_filter_name,
                   filter_path)
    except re.error as err:
      raise errors.ParserError(
          "Loading filter-file {} failed. Unable to"
          " compile regular expression value '{}'. Error {!r}".format(
              filter_path, filter_list["regex_match"], err))

  def _add_filters(self, filter_list, filter_path):
    """Adds filters loaded from file_name to list of filters to use.

    Args:
        filter_list(list): of filter dictionaries definitions to add
        filter_path(str): that provided the list of filters being added

    Raises:
        ParserError: when filters is not of type list
    """
    if not isinstance(filter_list, list):
      raise errors.ParserError(
          "Loading filter-file {} failed. "
          "Expecting value of 'filter_list' entry to be a list "
          "but instead its a {}.".format(filter_path, type(filter_list)))

    for cur_filter in filter_list:
      self._add_filter(cur_filter, filter_path)

  def _load_filter_directory(self, filter_path):
    """Loads every .json file found in filter_path specified.

    Args:
        filter_path(str): path to filter directory

    Raises:
        ParserError: if unable to access the filter_path provided

    Note:
        Only files ending in .json are loaded, others are skipped(see
        load_filter_file method for an example)
    """
    try:
      logger.debug("Adding filters from directory %s", filter_path)
      for filter_file in os.listdir(filter_path):
        if filter_file.endswith(".json"):
          filter_file_path = os.path.join(filter_path, filter_file)
          self.load_filter_file(filter_file_path)
        else:
          logger.debug("Skipping file %s missing .json extension", filter_path)
    except OSError:
      raise errors.ParserError(
          "Unable to access filter path '{}'".format(filter_path))

  @decorators.CapabilityLogDecorator(logger, level=decorators.DEBUG)
  def load_filters(self, filters: Collection[str]) -> None:
    """Load JSON filter files or directories specified.

    Args:
        filters: JSON filter files or paths to filter files.

    Raises:
        ParserError: if files or paths do not exist

    Note:
        Only files ending in .json are loaded, others are skipped(see
        load_filter_file method for an example)
    """
    for filter_path in filters:
      if not os.path.exists(filter_path):
        raise errors.ParserError(f"Filter path {filter_path!r} doesn't exist")
      elif os.path.isdir(filter_path):
        self._load_filter_directory(filter_path)
      else:
        self.load_filter_file(filter_path)

  def _nearest_datetime(self, datetime_list, target_datetime):
    """Locates a datetime.datetime closest to target datetime.

    Args:
        datetime_list(list): Containing datetime.datetime entries
        target_datetime(datetime.datetime): Desired target

    Returns:
        datetime.datetime: Closest entry to target datetime without
        exceeding target datetime

    Raises:
        ParserError: If datetime_list is empty
    """
    if not datetime_list:
      raise errors.ParserError(
          "Input parameter datetime_list length is zero. Required"
          " parameters: [datetime.datetime], datetime.datetime")
    work_list = [entry for entry in datetime_list if entry < target_datetime]
    if not work_list:
      raise errors.ParserError(
          "work_list length is zero. Entries in datetime_list"
          " {} are not < target_datetime {}".format(datetime_list,
                                                    target_datetime))
    return min(
        work_list,
        key=lambda datetime_entry: abs(datetime_entry - target_datetime))



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""ESP32 dev board device class."""
from gazoo_device import detect_criteria
from gazoo_device.base_classes import espressif_esp32_device


class ESP32(espressif_esp32_device.EspressifESP32Device):
  """Base Class for ESP32 Devices.

  ESP32 which includes Zigbee, WiFi and BLE to serve as a CHIP-compliant device,
  see more details in https://www.espressif.com/en/products/socs/esp32.
  """
  DETECT_MATCH_CRITERIA = {
      detect_criteria.PigweedQuery.IS_MATTER: False,
      detect_criteria.PigweedQuery.MANUFACTURER_NAME:
          r"(silicon(_| )labs|1a86)",
      detect_criteria.PigweedQuery.PRODUCT_NAME:
          r"(cp2104 usb to uart bridge controller|usb single serial)",
  }
  DEVICE_TYPE = "esp32"
  _OWNER_EMAIL = "gdm-authors@google.com"



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Interface for the Matter Color Control cluster capability."""

import abc
from gazoo_device.capabilities import matter_enums
from gazoo_device.capabilities.matter_clusters.interfaces import cluster_base


class ColorControlClusterBase(cluster_base.ClusterBase, metaclass=abc.ABCMeta):
  """Matter Color Control cluster capability."""

  CLUSTER_ID = matter_enums.ColorControlCluster.ID

  @abc.abstractmethod
  def move_to_hue(self, hue: int, verify: bool = True) -> None:
    """The MoveToHue command.

    Args:
      hue: The hue that the device should move to.
      verify: If true, verifies the hue changes before returning.
    """

  @abc.abstractmethod
  def move_to_saturation(self, saturation: int, verify: bool = True) -> None:
    """The MoveToSaturation command.

    Args:
      saturation: The saturation that the device should move to.
      verify: If true, verifies the saturation changes before returning.
    """

  @property
  @abc.abstractmethod
  def current_hue(self) -> int:
    """The CurrentHue attribute.

    Returns:
      The current hue.
    """

  @property
  @abc.abstractmethod
  def current_saturation(self) -> int:
    """The CurrentSaturation attribute.

    Returns:
      The current saturation.
    """

  # TODO(b/212193200)
  # Add additional attributes / commands for color control cluster



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: attributes_service.proto
# pylint: skip-file
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gazoo_device.protos import common_pb2 as common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='attributes_service.proto',
  package='chip.rpc',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x18\x61ttributes_service.proto\x12\x08\x63hip.rpc\x1a\x0c\x63ommon.proto\"\x8a\x01\n\x11\x41ttributeMetadata\x12\x10\n\x08\x65ndpoint\x18\x01 \x01(\r\x12&\n\x07\x63luster\x18\x02 \x01(\x0e\x32\x15.chip.rpc.ClusterType\x12\x14\n\x0c\x61ttribute_id\x18\x03 \x01(\r\x12%\n\x04type\x18\x04 \x01(\x0e\x32\x17.chip.rpc.AttributeType\"\xc7\x01\n\rAttributeData\x12\x13\n\tdata_bool\x18\x01 \x01(\x08H\x00\x12\x14\n\ndata_uint8\x18\x02 \x01(\rH\x00\x12\x15\n\x0b\x64\x61ta_uint16\x18\x03 \x01(\rH\x00\x12\x15\n\x0b\x64\x61ta_uint32\x18\x04 \x01(\rH\x00\x12\x14\n\ndata_bytes\x18\x05 \x01(\x0cH\x00\x12\x13\n\tdata_int8\x18\x06 \x01(\x05H\x00\x12\x14\n\ndata_int16\x18\x07 \x01(\x05H\x00\x12\x14\n\ndata_int32\x18\x08 \x01(\x05H\x00\x42\x06\n\x04\x64\x61ta\"f\n\x0e\x41ttributeWrite\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.chip.rpc.AttributeMetadata\x12%\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x17.chip.rpc.AttributeData*\xab\x10\n\rAttributeType\x12\x1e\n\x1aZCL_NO_DATA_ATTRIBUTE_TYPE\x10\x00\x12\x1e\n\x1aZCL_BOOLEAN_ATTRIBUTE_TYPE\x10\x10\x12\x1e\n\x1aZCL_BITMAP8_ATTRIBUTE_TYPE\x10\x18\x12\x1f\n\x1bZCL_BITMAP16_ATTRIBUTE_TYPE\x10\x19\x12\x1f\n\x1bZCL_BITMAP32_ATTRIBUTE_TYPE\x10\x1b\x12\x1f\n\x1bZCL_BITMAP64_ATTRIBUTE_TYPE\x10\x1f\x12\x1c\n\x18ZCL_INT8U_ATTRIBUTE_TYPE\x10 \x12\x1d\n\x19ZCL_INT16U_ATTRIBUTE_TYPE\x10!\x12\x1d\n\x19ZCL_INT24U_ATTRIBUTE_TYPE\x10\"\x12\x1d\n\x19ZCL_INT32U_ATTRIBUTE_TYPE\x10#\x12\x1d\n\x19ZCL_INT40U_ATTRIBUTE_TYPE\x10$\x12\x1d\n\x19ZCL_INT48U_ATTRIBUTE_TYPE\x10%\x12\x1d\n\x19ZCL_INT56U_ATTRIBUTE_TYPE\x10&\x12\x1d\n\x19ZCL_INT64U_ATTRIBUTE_TYPE\x10\'\x12\x1c\n\x18ZCL_INT8S_ATTRIBUTE_TYPE\x10(\x12\x1d\n\x19ZCL_INT16S_ATTRIBUTE_TYPE\x10)\x12\x1d\n\x19ZCL_INT24S_ATTRIBUTE_TYPE\x10*\x12\x1d\n\x19ZCL_INT32S_ATTRIBUTE_TYPE\x10+\x12\x1d\n\x19ZCL_INT40S_ATTRIBUTE_TYPE\x10,\x12\x1d\n\x19ZCL_INT48S_ATTRIBUTE_TYPE\x10-\x12\x1d\n\x19ZCL_INT56S_ATTRIBUTE_TYPE\x10.\x12\x1d\n\x19ZCL_INT64S_ATTRIBUTE_TYPE\x10/\x12\x1c\n\x18ZCL_ENUM8_ATTRIBUTE_TYPE\x10\x30\x12\x1d\n\x19ZCL_ENUM16_ATTRIBUTE_TYPE\x10\x31\x12\x1d\n\x19ZCL_SINGLE_ATTRIBUTE_TYPE\x10\x39\x12\x1d\n\x19ZCL_DOUBLE_ATTRIBUTE_TYPE\x10:\x12#\n\x1fZCL_OCTET_STRING_ATTRIBUTE_TYPE\x10\x41\x12\"\n\x1eZCL_CHAR_STRING_ATTRIBUTE_TYPE\x10\x42\x12(\n$ZCL_LONG_OCTET_STRING_ATTRIBUTE_TYPE\x10\x43\x12\'\n#ZCL_LONG_CHAR_STRING_ATTRIBUTE_TYPE\x10\x44\x12\x1c\n\x18ZCL_ARRAY_ATTRIBUTE_TYPE\x10H\x12\x1d\n\x19ZCL_STRUCT_ATTRIBUTE_TYPE\x10L\x12\x1b\n\x16ZCL_TOD_ATTRIBUTE_TYPE\x10\xe0\x01\x12\x1c\n\x17ZCL_DATE_ATTRIBUTE_TYPE\x10\xe1\x01\x12\x1b\n\x16ZCL_UTC_ATTRIBUTE_TYPE\x10\xe2\x01\x12 \n\x1bZCL_EPOCH_US_ATTRIBUTE_TYPE\x10\xe3\x01\x12\x1f\n\x1aZCL_EPOCH_S_ATTRIBUTE_TYPE\x10\xe4\x01\x12\"\n\x1dZCL_SYSTIME_US_ATTRIBUTE_TYPE\x10\xe5\x01\x12\x1f\n\x1aZCL_PERCENT_ATTRIBUTE_TYPE\x10\xe6\x01\x12%\n ZCL_PERCENT100THS_ATTRIBUTE_TYPE\x10\xe7\x01\x12\"\n\x1dZCL_CLUSTER_ID_ATTRIBUTE_TYPE\x10\xe8\x01\x12!\n\x1cZCL_ATTRIB_ID_ATTRIBUTE_TYPE\x10\xe9\x01\x12 \n\x1bZCL_FIELD_ID_ATTRIBUTE_TYPE\x10\xea\x01\x12 \n\x1bZCL_EVENT_ID_ATTRIBUTE_TYPE\x10\xeb\x01\x12\"\n\x1dZCL_COMMAND_ID_ATTRIBUTE_TYPE\x10\xec\x01\x12!\n\x1cZCL_ACTION_ID_ATTRIBUTE_TYPE\x10\xed\x01\x12 \n\x1bZCL_TRANS_ID_ATTRIBUTE_TYPE\x10\xef\x01\x12\x1f\n\x1aZCL_NODE_ID_ATTRIBUTE_TYPE\x10\xf0\x01\x12!\n\x1cZCL_VENDOR_ID_ATTRIBUTE_TYPE\x10\xf1\x01\x12\"\n\x1dZCL_DEVTYPE_ID_ATTRIBUTE_TYPE\x10\xf2\x01\x12!\n\x1cZCL_FABRIC_ID_ATTRIBUTE_TYPE\x10\xf3\x01\x12 \n\x1bZCL_GROUP_ID_ATTRIBUTE_TYPE\x10\xf4\x01\x12\x1e\n\x19ZCL_STATUS_ATTRIBUTE_TYPE\x10\xf5\x01\x12 \n\x1bZCL_DATA_VER_ATTRIBUTE_TYPE\x10\xf6\x01\x12 \n\x1bZCL_EVENT_NO_ATTRIBUTE_TYPE\x10\xf7\x01\x12#\n\x1eZCL_ENDPOINT_NO_ATTRIBUTE_TYPE\x10\xf8\x01\x12\"\n\x1dZCL_FABRIC_IDX_ATTRIBUTE_TYPE\x10\xf9\x01\x12\x1d\n\x18ZCL_IPADR_ATTRIBUTE_TYPE\x10\xfa\x01\x12\x1f\n\x1aZCL_IPV4ADR_ATTRIBUTE_TYPE\x10\xfb\x01\x12\x1f\n\x1aZCL_IPV6ADR_ATTRIBUTE_TYPE\x10\xfc\x01\x12\x1f\n\x1aZCL_IPV6PRE_ATTRIBUTE_TYPE\x10\xfd\x01\x12\x1d\n\x18ZCL_HWADR_ATTRIBUTE_TYPE\x10\xfe\x01\x12\x1f\n\x1aZCL_UNKNOWN_ATTRIBUTE_TYPE\x10\xff\x01*\xc5$\n\x0b\x43lusterType\x12\x1a\n\x16ZCL_INVALID_CLUSTER_ID\x10\x00\x12\x1f\n\x1bZCL_POWER_CONFIG_CLUSTER_ID\x10\x01\x12\x1e\n\x1aZCL_DEVICE_TEMP_CLUSTER_ID\x10\x02\x12\x1b\n\x17ZCL_IDENTIFY_CLUSTER_ID\x10\x03\x12\x19\n\x15ZCL_GROUPS_CLUSTER_ID\x10\x04\x12\x19\n\x15ZCL_SCENES_CLUSTER_ID\x10\x05\x12\x19\n\x15ZCL_ON_OFF_CLUSTER_ID\x10\x06\x12\'\n#ZCL_ON_OFF_SWITCH_CONFIG_CLUSTER_ID\x10\x07\x12 \n\x1cZCL_LEVEL_CONTROL_CLUSTER_ID\x10\x08\x12\x18\n\x14ZCL_ALARM_CLUSTER_ID\x10\t\x12\x17\n\x13ZCL_TIME_CLUSTER_ID\x10\n\x12%\n!ZCL_BINARY_INPUT_BASIC_CLUSTER_ID\x10\x0f\x12 \n\x1cZCL_POWER_PROFILE_CLUSTER_ID\x10\x1a\x12$\n ZCL_APPLIANCE_CONTROL_CLUSTER_ID\x10\x1b\x12\x1d\n\x19ZCL_DESCRIPTOR_CLUSTER_ID\x10\x1d\x12\x1f\n\x1bZCL_POLL_CONTROL_CLUSTER_ID\x10 \x12\"\n\x1eZCL_BRIDGED_ACTIONS_CLUSTER_ID\x10%\x12\x18\n\x14ZCL_BASIC_CLUSTER_ID\x10(\x12\x1f\n\x1bZCL_OTA_PROVIDER_CLUSTER_ID\x10)\x12 \n\x1cZCL_OTA_REQUESTOR_CLUSTER_ID\x10*\x12$\n ZCL_UNIT_LOCALIZATION_CLUSTER_ID\x10-\x12\x1f\n\x1bZCL_POWER_SOURCE_CLUSTER_ID\x10/\x12(\n$ZCL_GENERAL_COMMISSIONING_CLUSTER_ID\x10\x30\x12(\n$ZCL_NETWORK_COMMISSIONING_CLUSTER_ID\x10\x31\x12\"\n\x1eZCL_DIAGNOSTIC_LOGS_CLUSTER_ID\x10\x32\x12&\n\"ZCL_GENERAL_DIAGNOSTICS_CLUSTER_ID\x10\x33\x12\'\n#ZCL_SOFTWARE_DIAGNOSTICS_CLUSTER_ID\x10\x34\x12-\n)ZCL_THREAD_NETWORK_DIAGNOSTICS_CLUSTER_ID\x10\x35\x12+\n\'ZCL_WIFI_NETWORK_DIAGNOSTICS_CLUSTER_ID\x10\x36\x12/\n+ZCL_ETHERNET_NETWORK_DIAGNOSTICS_CLUSTER_ID\x10\x37\x12\'\n#ZCL_BRIDGED_DEVICE_BASIC_CLUSTER_ID\x10\x39\x12\x19\n\x15ZCL_SWITCH_CLUSTER_ID\x10;\x12.\n*ZCL_ADMINISTRATOR_COMMISSIONING_CLUSTER_ID\x10<\x12*\n&ZCL_OPERATIONAL_CREDENTIALS_CLUSTER_ID\x10>\x12\x1e\n\x1aZCL_FIXED_LABEL_CLUSTER_ID\x10@\x12 \n\x1cZCL_BOOLEAN_STATE_CLUSTER_ID\x10\x45\x12\x1e\n\x1aZCL_MODE_SELECT_CLUSTER_ID\x10P\x12 \n\x1bZCL_SHADE_CONFIG_CLUSTER_ID\x10\x80\x02\x12\x1d\n\x18ZCL_DOOR_LOCK_CLUSTER_ID\x10\x81\x02\x12#\n\x1eZCL_WINDOW_COVERING_CLUSTER_ID\x10\x82\x02\x12#\n\x1eZCL_BARRIER_CONTROL_CLUSTER_ID\x10\x83\x02\x12\'\n\"ZCL_PUMP_CONFIG_CONTROL_CLUSTER_ID\x10\x80\x04\x12\x1e\n\x19ZCL_THERMOSTAT_CLUSTER_ID\x10\x81\x04\x12\x1f\n\x1aZCL_FAN_CONTROL_CLUSTER_ID\x10\x82\x04\x12#\n\x1eZCL_DEHUMID_CONTROL_CLUSTER_ID\x10\x83\x04\x12(\n#ZCL_THERMOSTAT_UI_CONFIG_CLUSTER_ID\x10\x84\x04\x12!\n\x1cZCL_COLOR_CONTROL_CLUSTER_ID\x10\x80\x06\x12)\n$ZCL_BALLAST_CONFIGURATION_CLUSTER_ID\x10\x81\x06\x12+\n&ZCL_ILLUMINANCE_MEASUREMENT_CLUSTER_ID\x10\x80\x08\x12$\n\x1fZCL_TEMP_MEASUREMENT_CLUSTER_ID\x10\x82\x08\x12(\n#ZCL_PRESSURE_MEASUREMENT_CLUSTER_ID\x10\x83\x08\x12$\n\x1fZCL_FLOW_MEASUREMENT_CLUSTER_ID\x10\x84\x08\x12\x31\n,ZCL_RELATIVE_HUMIDITY_MEASUREMENT_CLUSTER_ID\x10\x85\x08\x12%\n ZCL_OCCUPANCY_SENSING_CLUSTER_ID\x10\x86\x08\x12=\n8ZCL_CARBON_MONOXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x8c\x08\x12<\n7ZCL_CARBON_DIOXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x8d\x08\x12\x36\n1ZCL_ETHYLENE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x8e\x08\x12<\n7ZCL_ETHYLENE_OXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x8f\x08\x12\x36\n1ZCL_HYDROGEN_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x90\x08\x12?\n:ZCL_HYDROGEN_SULPHIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x91\x08\x12:\n5ZCL_NITRIC_OXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x92\x08\x12>\n9ZCL_NITROGEN_DIOXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x93\x08\x12\x34\n/ZCL_OXYGEN_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x94\x08\x12\x33\n.ZCL_OZONE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x95\x08\x12<\n7ZCL_SULFUR_DIOXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x96\x08\x12>\n9ZCL_DISSOLVED_OXYGEN_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x97\x08\x12\x35\n0ZCL_BROMATE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x98\x08\x12\x39\n4ZCL_CHLORAMINES_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x99\x08\x12\x36\n1ZCL_CHLORINE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x9a\x08\x12G\nBZCL_FECAL_COLIFORM_AND_E_COLI_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x9b\x08\x12\x36\n1ZCL_FLUORIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x9c\x08\x12>\n9ZCL_HALOACETIC_ACIDS_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x9d\x08\x12\x43\n>ZCL_TOTAL_TRIHALOMETHANES_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x9e\x08\x12\x45\n@ZCL_TOTAL_COLIFORM_BACTERIA_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\x9f\x08\x12\x37\n2ZCL_TURBIDITY_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\xa0\x08\x12\x34\n/ZCL_COPPER_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\xa1\x08\x12\x32\n-ZCL_LEAD_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\xa2\x08\x12\x37\n2ZCL_MANGANESE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\xa3\x08\x12\x35\n0ZCL_SULFATE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\xa4\x08\x12\x42\n=ZCL_BROMODICHLOROMETHANE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\xa5\x08\x12\x37\n2ZCL_BROMOFORM_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\xa6\x08\x12\x42\n=ZCL_CHLORODIBROMOMETHANE_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\xa7\x08\x12\x38\n3ZCL_CHLOROFORM_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\xa8\x08\x12\x34\n/ZCL_SODIUM_CONCENTRATION_MEASUREMENT_CLUSTER_ID\x10\xa9\x08\x12\x1c\n\x17ZCL_IAS_ZONE_CLUSTER_ID\x10\x80\n\x12\x1b\n\x16ZCL_IAS_ACE_CLUSTER_ID\x10\x81\n\x12\x1a\n\x15ZCL_IAS_WD_CLUSTER_ID\x10\x82\n\x12\x1f\n\x1aZCL_WAKE_ON_LAN_CLUSTER_ID\x10\x83\n\x12\x1b\n\x16ZCL_CHANNEL_CLUSTER_ID\x10\x84\n\x12$\n\x1fZCL_TARGET_NAVIGATOR_CLUSTER_ID\x10\x85\n\x12\"\n\x1dZCL_MEDIA_PLAYBACK_CLUSTER_ID\x10\x86\n\x12\x1f\n\x1aZCL_MEDIA_INPUT_CLUSTER_ID\x10\x87\n\x12\x1d\n\x18ZCL_LOW_POWER_CLUSTER_ID\x10\x88\n\x12 \n\x1bZCL_KEYPAD_INPUT_CLUSTER_ID\x10\x89\n\x12\"\n\x1dZCL_CONTENT_LAUNCH_CLUSTER_ID\x10\x8a\n\x12 \n\x1bZCL_AUDIO_OUTPUT_CLUSTER_ID\x10\x8b\n\x12(\n#ZCL_APPLICATION_LAUNCHER_CLUSTER_ID\x10\x8c\n\x12%\n ZCL_APPLICATION_BASIC_CLUSTER_ID\x10\x8d\n\x12!\n\x1cZCL_ACCOUNT_LOGIN_CLUSTER_ID\x10\x8e\n\x12\x18\n\x13ZCL_TEST_CLUSTER_ID\x10\x8f\n\x12\x1d\n\x18ZCL_MESSAGING_CLUSTER_ID\x10\x83\x0e\x12,\n\'ZCL_APPLIANCE_IDENTIFICATION_CLUSTER_ID\x10\x80\x16\x12(\n#ZCL_METER_IDENTIFICATION_CLUSTER_ID\x10\x81\x16\x12.\n)ZCL_APPLIANCE_EVENTS_AND_ALERT_CLUSTER_ID\x10\x82\x16\x12(\n#ZCL_APPLIANCE_STATISTICS_CLUSTER_ID\x10\x83\x16\x12*\n%ZCL_ELECTRICAL_MEASUREMENT_CLUSTER_ID\x10\x84\x16\x12\x1c\n\x16ZCL_BINDING_CLUSTER_ID\x10\x80\xe0\x03\x12)\n#ZCL_GROUP_KEY_MANAGEMENT_CLUSTER_ID\x10\x84\xe0\x03\x12(\n\"ZCL_SAMPLE_MFG_SPECIFIC_CLUSTER_ID\x10\x80\xf8\x03\x32\x85\x01\n\nAttributes\x12\x37\n\x05Write\x12\x18.chip.rpc.AttributeWrite\x1a\x12.pw.protobuf.Empty\"\x00\x12>\n\x04Read\x12\x1b.chip.rpc.AttributeMetadata\x1a\x17.chip.rpc.AttributeData\"\x00\x62\x06proto3'
  ,
  dependencies=[common__pb2.DESCRIPTOR,])

_ATTRIBUTETYPE = _descriptor.EnumDescriptor(
  name='AttributeType',
  full_name='chip.rpc.AttributeType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ZCL_NO_DATA_ATTRIBUTE_TYPE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BOOLEAN_ATTRIBUTE_TYPE', index=1, number=16,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BITMAP8_ATTRIBUTE_TYPE', index=2, number=24,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BITMAP16_ATTRIBUTE_TYPE', index=3, number=25,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BITMAP32_ATTRIBUTE_TYPE', index=4, number=27,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BITMAP64_ATTRIBUTE_TYPE', index=5, number=31,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT8U_ATTRIBUTE_TYPE', index=6, number=32,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT16U_ATTRIBUTE_TYPE', index=7, number=33,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT24U_ATTRIBUTE_TYPE', index=8, number=34,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT32U_ATTRIBUTE_TYPE', index=9, number=35,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT40U_ATTRIBUTE_TYPE', index=10, number=36,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT48U_ATTRIBUTE_TYPE', index=11, number=37,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT56U_ATTRIBUTE_TYPE', index=12, number=38,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT64U_ATTRIBUTE_TYPE', index=13, number=39,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT8S_ATTRIBUTE_TYPE', index=14, number=40,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT16S_ATTRIBUTE_TYPE', index=15, number=41,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT24S_ATTRIBUTE_TYPE', index=16, number=42,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT32S_ATTRIBUTE_TYPE', index=17, number=43,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT40S_ATTRIBUTE_TYPE', index=18, number=44,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT48S_ATTRIBUTE_TYPE', index=19, number=45,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT56S_ATTRIBUTE_TYPE', index=20, number=46,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_INT64S_ATTRIBUTE_TYPE', index=21, number=47,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ENUM8_ATTRIBUTE_TYPE', index=22, number=48,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ENUM16_ATTRIBUTE_TYPE', index=23, number=49,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_SINGLE_ATTRIBUTE_TYPE', index=24, number=57,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_DOUBLE_ATTRIBUTE_TYPE', index=25, number=58,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_OCTET_STRING_ATTRIBUTE_TYPE', index=26, number=65,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_CHAR_STRING_ATTRIBUTE_TYPE', index=27, number=66,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_LONG_OCTET_STRING_ATTRIBUTE_TYPE', index=28, number=67,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_LONG_CHAR_STRING_ATTRIBUTE_TYPE', index=29, number=68,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ARRAY_ATTRIBUTE_TYPE', index=30, number=72,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_STRUCT_ATTRIBUTE_TYPE', index=31, number=76,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_TOD_ATTRIBUTE_TYPE', index=32, number=224,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_DATE_ATTRIBUTE_TYPE', index=33, number=225,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_UTC_ATTRIBUTE_TYPE', index=34, number=226,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_EPOCH_US_ATTRIBUTE_TYPE', index=35, number=227,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_EPOCH_S_ATTRIBUTE_TYPE', index=36, number=228,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_SYSTIME_US_ATTRIBUTE_TYPE', index=37, number=229,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_PERCENT_ATTRIBUTE_TYPE', index=38, number=230,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_PERCENT100THS_ATTRIBUTE_TYPE', index=39, number=231,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_CLUSTER_ID_ATTRIBUTE_TYPE', index=40, number=232,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ATTRIB_ID_ATTRIBUTE_TYPE', index=41, number=233,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_FIELD_ID_ATTRIBUTE_TYPE', index=42, number=234,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_EVENT_ID_ATTRIBUTE_TYPE', index=43, number=235,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_COMMAND_ID_ATTRIBUTE_TYPE', index=44, number=236,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ACTION_ID_ATTRIBUTE_TYPE', index=45, number=237,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_TRANS_ID_ATTRIBUTE_TYPE', index=46, number=239,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_NODE_ID_ATTRIBUTE_TYPE', index=47, number=240,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_VENDOR_ID_ATTRIBUTE_TYPE', index=48, number=241,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_DEVTYPE_ID_ATTRIBUTE_TYPE', index=49, number=242,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_FABRIC_ID_ATTRIBUTE_TYPE', index=50, number=243,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_GROUP_ID_ATTRIBUTE_TYPE', index=51, number=244,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_STATUS_ATTRIBUTE_TYPE', index=52, number=245,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_DATA_VER_ATTRIBUTE_TYPE', index=53, number=246,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_EVENT_NO_ATTRIBUTE_TYPE', index=54, number=247,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ENDPOINT_NO_ATTRIBUTE_TYPE', index=55, number=248,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_FABRIC_IDX_ATTRIBUTE_TYPE', index=56, number=249,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_IPADR_ATTRIBUTE_TYPE', index=57, number=250,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_IPV4ADR_ATTRIBUTE_TYPE', index=58, number=251,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_IPV6ADR_ATTRIBUTE_TYPE', index=59, number=252,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_IPV6PRE_ATTRIBUTE_TYPE', index=60, number=253,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_HWADR_ATTRIBUTE_TYPE', index=61, number=254,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_UNKNOWN_ATTRIBUTE_TYPE', index=62, number=255,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=500,
  serialized_end=2591,
)
_sym_db.RegisterEnumDescriptor(_ATTRIBUTETYPE)

AttributeType = enum_type_wrapper.EnumTypeWrapper(_ATTRIBUTETYPE)
_CLUSTERTYPE = _descriptor.EnumDescriptor(
  name='ClusterType',
  full_name='chip.rpc.ClusterType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ZCL_INVALID_CLUSTER_ID', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_POWER_CONFIG_CLUSTER_ID', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_DEVICE_TEMP_CLUSTER_ID', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_IDENTIFY_CLUSTER_ID', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_GROUPS_CLUSTER_ID', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_SCENES_CLUSTER_ID', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ON_OFF_CLUSTER_ID', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ON_OFF_SWITCH_CONFIG_CLUSTER_ID', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_LEVEL_CONTROL_CLUSTER_ID', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ALARM_CLUSTER_ID', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_TIME_CLUSTER_ID', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BINARY_INPUT_BASIC_CLUSTER_ID', index=11, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_POWER_PROFILE_CLUSTER_ID', index=12, number=26,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_APPLIANCE_CONTROL_CLUSTER_ID', index=13, number=27,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_DESCRIPTOR_CLUSTER_ID', index=14, number=29,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_POLL_CONTROL_CLUSTER_ID', index=15, number=32,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BRIDGED_ACTIONS_CLUSTER_ID', index=16, number=37,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BASIC_CLUSTER_ID', index=17, number=40,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_OTA_PROVIDER_CLUSTER_ID', index=18, number=41,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_OTA_REQUESTOR_CLUSTER_ID', index=19, number=42,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_UNIT_LOCALIZATION_CLUSTER_ID', index=20, number=45,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_POWER_SOURCE_CLUSTER_ID', index=21, number=47,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_GENERAL_COMMISSIONING_CLUSTER_ID', index=22, number=48,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_NETWORK_COMMISSIONING_CLUSTER_ID', index=23, number=49,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_DIAGNOSTIC_LOGS_CLUSTER_ID', index=24, number=50,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_GENERAL_DIAGNOSTICS_CLUSTER_ID', index=25, number=51,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_SOFTWARE_DIAGNOSTICS_CLUSTER_ID', index=26, number=52,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_THREAD_NETWORK_DIAGNOSTICS_CLUSTER_ID', index=27, number=53,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_WIFI_NETWORK_DIAGNOSTICS_CLUSTER_ID', index=28, number=54,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ETHERNET_NETWORK_DIAGNOSTICS_CLUSTER_ID', index=29, number=55,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BRIDGED_DEVICE_BASIC_CLUSTER_ID', index=30, number=57,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_SWITCH_CLUSTER_ID', index=31, number=59,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ADMINISTRATOR_COMMISSIONING_CLUSTER_ID', index=32, number=60,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_OPERATIONAL_CREDENTIALS_CLUSTER_ID', index=33, number=62,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_FIXED_LABEL_CLUSTER_ID', index=34, number=64,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BOOLEAN_STATE_CLUSTER_ID', index=35, number=69,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_MODE_SELECT_CLUSTER_ID', index=36, number=80,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_SHADE_CONFIG_CLUSTER_ID', index=37, number=256,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_DOOR_LOCK_CLUSTER_ID', index=38, number=257,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_WINDOW_COVERING_CLUSTER_ID', index=39, number=258,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BARRIER_CONTROL_CLUSTER_ID', index=40, number=259,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_PUMP_CONFIG_CONTROL_CLUSTER_ID', index=41, number=512,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_THERMOSTAT_CLUSTER_ID', index=42, number=513,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_FAN_CONTROL_CLUSTER_ID', index=43, number=514,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_DEHUMID_CONTROL_CLUSTER_ID', index=44, number=515,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_THERMOSTAT_UI_CONFIG_CLUSTER_ID', index=45, number=516,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_COLOR_CONTROL_CLUSTER_ID', index=46, number=768,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BALLAST_CONFIGURATION_CLUSTER_ID', index=47, number=769,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ILLUMINANCE_MEASUREMENT_CLUSTER_ID', index=48, number=1024,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_TEMP_MEASUREMENT_CLUSTER_ID', index=49, number=1026,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_PRESSURE_MEASUREMENT_CLUSTER_ID', index=50, number=1027,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_FLOW_MEASUREMENT_CLUSTER_ID', index=51, number=1028,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_RELATIVE_HUMIDITY_MEASUREMENT_CLUSTER_ID', index=52, number=1029,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_OCCUPANCY_SENSING_CLUSTER_ID', index=53, number=1030,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_CARBON_MONOXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=54, number=1036,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_CARBON_DIOXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=55, number=1037,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ETHYLENE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=56, number=1038,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ETHYLENE_OXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=57, number=1039,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_HYDROGEN_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=58, number=1040,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_HYDROGEN_SULPHIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=59, number=1041,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_NITRIC_OXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=60, number=1042,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_NITROGEN_DIOXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=61, number=1043,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_OXYGEN_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=62, number=1044,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_OZONE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=63, number=1045,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_SULFUR_DIOXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=64, number=1046,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_DISSOLVED_OXYGEN_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=65, number=1047,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BROMATE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=66, number=1048,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_CHLORAMINES_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=67, number=1049,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_CHLORINE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=68, number=1050,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_FECAL_COLIFORM_AND_E_COLI_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=69, number=1051,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_FLUORIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=70, number=1052,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_HALOACETIC_ACIDS_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=71, number=1053,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_TOTAL_TRIHALOMETHANES_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=72, number=1054,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_TOTAL_COLIFORM_BACTERIA_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=73, number=1055,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_TURBIDITY_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=74, number=1056,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_COPPER_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=75, number=1057,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_LEAD_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=76, number=1058,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_MANGANESE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=77, number=1059,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_SULFATE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=78, number=1060,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BROMODICHLOROMETHANE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=79, number=1061,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BROMOFORM_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=80, number=1062,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_CHLORODIBROMOMETHANE_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=81, number=1063,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_CHLOROFORM_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=82, number=1064,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_SODIUM_CONCENTRATION_MEASUREMENT_CLUSTER_ID', index=83, number=1065,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_IAS_ZONE_CLUSTER_ID', index=84, number=1280,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_IAS_ACE_CLUSTER_ID', index=85, number=1281,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_IAS_WD_CLUSTER_ID', index=86, number=1282,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_WAKE_ON_LAN_CLUSTER_ID', index=87, number=1283,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_CHANNEL_CLUSTER_ID', index=88, number=1284,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_TARGET_NAVIGATOR_CLUSTER_ID', index=89, number=1285,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_MEDIA_PLAYBACK_CLUSTER_ID', index=90, number=1286,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_MEDIA_INPUT_CLUSTER_ID', index=91, number=1287,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_LOW_POWER_CLUSTER_ID', index=92, number=1288,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_KEYPAD_INPUT_CLUSTER_ID', index=93, number=1289,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_CONTENT_LAUNCH_CLUSTER_ID', index=94, number=1290,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_AUDIO_OUTPUT_CLUSTER_ID', index=95, number=1291,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_APPLICATION_LAUNCHER_CLUSTER_ID', index=96, number=1292,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_APPLICATION_BASIC_CLUSTER_ID', index=97, number=1293,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ACCOUNT_LOGIN_CLUSTER_ID', index=98, number=1294,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_TEST_CLUSTER_ID', index=99, number=1295,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_MESSAGING_CLUSTER_ID', index=100, number=1795,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_APPLIANCE_IDENTIFICATION_CLUSTER_ID', index=101, number=2816,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_METER_IDENTIFICATION_CLUSTER_ID', index=102, number=2817,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_APPLIANCE_EVENTS_AND_ALERT_CLUSTER_ID', index=103, number=2818,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_APPLIANCE_STATISTICS_CLUSTER_ID', index=104, number=2819,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_ELECTRICAL_MEASUREMENT_CLUSTER_ID', index=105, number=2820,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_BINDING_CLUSTER_ID', index=106, number=61440,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_GROUP_KEY_MANAGEMENT_CLUSTER_ID', index=107, number=61444,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ZCL_SAMPLE_MFG_SPECIFIC_CLUSTER_ID', index=108, number=64512,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2594,
  serialized_end=7271,
)
_sym_db.RegisterEnumDescriptor(_CLUSTERTYPE)

ClusterType = enum_type_wrapper.EnumTypeWrapper(_CLUSTERTYPE)
ZCL_NO_DATA_ATTRIBUTE_TYPE = 0
ZCL_BOOLEAN_ATTRIBUTE_TYPE = 16
ZCL_BITMAP8_ATTRIBUTE_TYPE = 24
ZCL_BITMAP16_ATTRIBUTE_TYPE = 25
ZCL_BITMAP32_ATTRIBUTE_TYPE = 27
ZCL_BITMAP64_ATTRIBUTE_TYPE = 31
ZCL_INT8U_ATTRIBUTE_TYPE = 32
ZCL_INT16U_ATTRIBUTE_TYPE = 33
ZCL_INT24U_ATTRIBUTE_TYPE = 34
ZCL_INT32U_ATTRIBUTE_TYPE = 35
ZCL_INT40U_ATTRIBUTE_TYPE = 36
ZCL_INT48U_ATTRIBUTE_TYPE = 37
ZCL_INT56U_ATTRIBUTE_TYPE = 38
ZCL_INT64U_ATTRIBUTE_TYPE = 39
ZCL_INT8S_ATTRIBUTE_TYPE = 40
ZCL_INT16S_ATTRIBUTE_TYPE = 41
ZCL_INT24S_ATTRIBUTE_TYPE = 42
ZCL_INT32S_ATTRIBUTE_TYPE = 43
ZCL_INT40S_ATTRIBUTE_TYPE = 44
ZCL_INT48S_ATTRIBUTE_TYPE = 45
ZCL_INT56S_ATTRIBUTE_TYPE = 46
ZCL_INT64S_ATTRIBUTE_TYPE = 47
ZCL_ENUM8_ATTRIBUTE_TYPE = 48
ZCL_ENUM16_ATTRIBUTE_TYPE = 49
ZCL_SINGLE_ATTRIBUTE_TYPE = 57
ZCL_DOUBLE_ATTRIBUTE_TYPE = 58
ZCL_OCTET_STRING_ATTRIBUTE_TYPE = 65
ZCL_CHAR_STRING_ATTRIBUTE_TYPE = 66
ZCL_LONG_OCTET_STRING_ATTRIBUTE_TYPE = 67
ZCL_LONG_CHAR_STRING_ATTRIBUTE_TYPE = 68
ZCL_ARRAY_ATTRIBUTE_TYPE = 72
ZCL_STRUCT_ATTRIBUTE_TYPE = 76
ZCL_TOD_ATTRIBUTE_TYPE = 224
ZCL_DATE_ATTRIBUTE_TYPE = 225
ZCL_UTC_ATTRIBUTE_TYPE = 226
ZCL_EPOCH_US_ATTRIBUTE_TYPE = 227
ZCL_EPOCH_S_ATTRIBUTE_TYPE = 228
ZCL_SYSTIME_US_ATTRIBUTE_TYPE = 229
ZCL_PERCENT_ATTRIBUTE_TYPE = 230
ZCL_PERCENT100THS_ATTRIBUTE_TYPE = 231
ZCL_CLUSTER_ID_ATTRIBUTE_TYPE = 232
ZCL_ATTRIB_ID_ATTRIBUTE_TYPE = 233
ZCL_FIELD_ID_ATTRIBUTE_TYPE = 234
ZCL_EVENT_ID_ATTRIBUTE_TYPE = 235
ZCL_COMMAND_ID_ATTRIBUTE_TYPE = 236
ZCL_ACTION_ID_ATTRIBUTE_TYPE = 237
ZCL_TRANS_ID_ATTRIBUTE_TYPE = 239
ZCL_NODE_ID_ATTRIBUTE_TYPE = 240
ZCL_VENDOR_ID_ATTRIBUTE_TYPE = 241
ZCL_DEVTYPE_ID_ATTRIBUTE_TYPE = 242
ZCL_FABRIC_ID_ATTRIBUTE_TYPE = 243
ZCL_GROUP_ID_ATTRIBUTE_TYPE = 244
ZCL_STATUS_ATTRIBUTE_TYPE = 245
ZCL_DATA_VER_ATTRIBUTE_TYPE = 246
ZCL_EVENT_NO_ATTRIBUTE_TYPE = 247
ZCL_ENDPOINT_NO_ATTRIBUTE_TYPE = 248
ZCL_FABRIC_IDX_ATTRIBUTE_TYPE = 249
ZCL_IPADR_ATTRIBUTE_TYPE = 250
ZCL_IPV4ADR_ATTRIBUTE_TYPE = 251
ZCL_IPV6ADR_ATTRIBUTE_TYPE = 252
ZCL_IPV6PRE_ATTRIBUTE_TYPE = 253
ZCL_HWADR_ATTRIBUTE_TYPE = 254
ZCL_UNKNOWN_ATTRIBUTE_TYPE = 255
ZCL_INVALID_CLUSTER_ID = 0
ZCL_POWER_CONFIG_CLUSTER_ID = 1
ZCL_DEVICE_TEMP_CLUSTER_ID = 2
ZCL_IDENTIFY_CLUSTER_ID = 3
ZCL_GROUPS_CLUSTER_ID = 4
ZCL_SCENES_CLUSTER_ID = 5
ZCL_ON_OFF_CLUSTER_ID = 6
ZCL_ON_OFF_SWITCH_CONFIG_CLUSTER_ID = 7
ZCL_LEVEL_CONTROL_CLUSTER_ID = 8
ZCL_ALARM_CLUSTER_ID = 9
ZCL_TIME_CLUSTER_ID = 10
ZCL_BINARY_INPUT_BASIC_CLUSTER_ID = 15
ZCL_POWER_PROFILE_CLUSTER_ID = 26
ZCL_APPLIANCE_CONTROL_CLUSTER_ID = 27
ZCL_DESCRIPTOR_CLUSTER_ID = 29
ZCL_POLL_CONTROL_CLUSTER_ID = 32
ZCL_BRIDGED_ACTIONS_CLUSTER_ID = 37
ZCL_BASIC_CLUSTER_ID = 40
ZCL_OTA_PROVIDER_CLUSTER_ID = 41
ZCL_OTA_REQUESTOR_CLUSTER_ID = 42
ZCL_UNIT_LOCALIZATION_CLUSTER_ID = 45
ZCL_POWER_SOURCE_CLUSTER_ID = 47
ZCL_GENERAL_COMMISSIONING_CLUSTER_ID = 48
ZCL_NETWORK_COMMISSIONING_CLUSTER_ID = 49
ZCL_DIAGNOSTIC_LOGS_CLUSTER_ID = 50
ZCL_GENERAL_DIAGNOSTICS_CLUSTER_ID = 51
ZCL_SOFTWARE_DIAGNOSTICS_CLUSTER_ID = 52
ZCL_THREAD_NETWORK_DIAGNOSTICS_CLUSTER_ID = 53
ZCL_WIFI_NETWORK_DIAGNOSTICS_CLUSTER_ID = 54
ZCL_ETHERNET_NETWORK_DIAGNOSTICS_CLUSTER_ID = 55
ZCL_BRIDGED_DEVICE_BASIC_CLUSTER_ID = 57
ZCL_SWITCH_CLUSTER_ID = 59
ZCL_ADMINISTRATOR_COMMISSIONING_CLUSTER_ID = 60
ZCL_OPERATIONAL_CREDENTIALS_CLUSTER_ID = 62
ZCL_FIXED_LABEL_CLUSTER_ID = 64
ZCL_BOOLEAN_STATE_CLUSTER_ID = 69
ZCL_MODE_SELECT_CLUSTER_ID = 80
ZCL_SHADE_CONFIG_CLUSTER_ID = 256
ZCL_DOOR_LOCK_CLUSTER_ID = 257
ZCL_WINDOW_COVERING_CLUSTER_ID = 258
ZCL_BARRIER_CONTROL_CLUSTER_ID = 259
ZCL_PUMP_CONFIG_CONTROL_CLUSTER_ID = 512
ZCL_THERMOSTAT_CLUSTER_ID = 513
ZCL_FAN_CONTROL_CLUSTER_ID = 514
ZCL_DEHUMID_CONTROL_CLUSTER_ID = 515
ZCL_THERMOSTAT_UI_CONFIG_CLUSTER_ID = 516
ZCL_COLOR_CONTROL_CLUSTER_ID = 768
ZCL_BALLAST_CONFIGURATION_CLUSTER_ID = 769
ZCL_ILLUMINANCE_MEASUREMENT_CLUSTER_ID = 1024
ZCL_TEMP_MEASUREMENT_CLUSTER_ID = 1026
ZCL_PRESSURE_MEASUREMENT_CLUSTER_ID = 1027
ZCL_FLOW_MEASUREMENT_CLUSTER_ID = 1028
ZCL_RELATIVE_HUMIDITY_MEASUREMENT_CLUSTER_ID = 1029
ZCL_OCCUPANCY_SENSING_CLUSTER_ID = 1030
ZCL_CARBON_MONOXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1036
ZCL_CARBON_DIOXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1037
ZCL_ETHYLENE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1038
ZCL_ETHYLENE_OXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1039
ZCL_HYDROGEN_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1040
ZCL_HYDROGEN_SULPHIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1041
ZCL_NITRIC_OXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1042
ZCL_NITROGEN_DIOXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1043
ZCL_OXYGEN_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1044
ZCL_OZONE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1045
ZCL_SULFUR_DIOXIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1046
ZCL_DISSOLVED_OXYGEN_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1047
ZCL_BROMATE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1048
ZCL_CHLORAMINES_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1049
ZCL_CHLORINE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1050
ZCL_FECAL_COLIFORM_AND_E_COLI_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1051
ZCL_FLUORIDE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1052
ZCL_HALOACETIC_ACIDS_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1053
ZCL_TOTAL_TRIHALOMETHANES_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1054
ZCL_TOTAL_COLIFORM_BACTERIA_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1055
ZCL_TURBIDITY_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1056
ZCL_COPPER_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1057
ZCL_LEAD_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1058
ZCL_MANGANESE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1059
ZCL_SULFATE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1060
ZCL_BROMODICHLOROMETHANE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1061
ZCL_BROMOFORM_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1062
ZCL_CHLORODIBROMOMETHANE_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1063
ZCL_CHLOROFORM_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1064
ZCL_SODIUM_CONCENTRATION_MEASUREMENT_CLUSTER_ID = 1065
ZCL_IAS_ZONE_CLUSTER_ID = 1280
ZCL_IAS_ACE_CLUSTER_ID = 1281
ZCL_IAS_WD_CLUSTER_ID = 1282
ZCL_WAKE_ON_LAN_CLUSTER_ID = 1283
ZCL_CHANNEL_CLUSTER_ID = 1284
ZCL_TARGET_NAVIGATOR_CLUSTER_ID = 1285
ZCL_MEDIA_PLAYBACK_CLUSTER_ID = 1286
ZCL_MEDIA_INPUT_CLUSTER_ID = 1287
ZCL_LOW_POWER_CLUSTER_ID = 1288
ZCL_KEYPAD_INPUT_CLUSTER_ID = 1289
ZCL_CONTENT_LAUNCH_CLUSTER_ID = 1290
ZCL_AUDIO_OUTPUT_CLUSTER_ID = 1291
ZCL_APPLICATION_LAUNCHER_CLUSTER_ID = 1292
ZCL_APPLICATION_BASIC_CLUSTER_ID = 1293
ZCL_ACCOUNT_LOGIN_CLUSTER_ID = 1294
ZCL_TEST_CLUSTER_ID = 1295
ZCL_MESSAGING_CLUSTER_ID = 1795
ZCL_APPLIANCE_IDENTIFICATION_CLUSTER_ID = 2816
ZCL_METER_IDENTIFICATION_CLUSTER_ID = 2817
ZCL_APPLIANCE_EVENTS_AND_ALERT_CLUSTER_ID = 2818
ZCL_APPLIANCE_STATISTICS_CLUSTER_ID = 2819
ZCL_ELECTRICAL_MEASUREMENT_CLUSTER_ID = 2820
ZCL_BINDING_CLUSTER_ID = 61440
ZCL_GROUP_KEY_MANAGEMENT_CLUSTER_ID = 61444
ZCL_SAMPLE_MFG_SPECIFIC_CLUSTER_ID = 64512



_ATTRIBUTEMETADATA = _descriptor.Descriptor(
  name='AttributeMetadata',
  full_name='chip.rpc.AttributeMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='chip.rpc.AttributeMetadata.endpoint', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cluster', full_name='chip.rpc.AttributeMetadata.cluster', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='attribute_id', full_name='chip.rpc.AttributeMetadata.attribute_id', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='chip.rpc.AttributeMetadata.type', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=53,
  serialized_end=191,
)


_ATTRIBUTEDATA = _descriptor.Descriptor(
  name='AttributeData',
  full_name='chip.rpc.AttributeData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data_bool', full_name='chip.rpc.AttributeData.data_bool', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_uint8', full_name='chip.rpc.AttributeData.data_uint8', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_uint16', full_name='chip.rpc.AttributeData.data_uint16', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_uint32', full_name='chip.rpc.AttributeData.data_uint32', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_bytes', full_name='chip.rpc.AttributeData.data_bytes', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_int8', full_name='chip.rpc.AttributeData.data_int8', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_int16', full_name='chip.rpc.AttributeData.data_int16', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_int32', full_name='chip.rpc.AttributeData.data_int32', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='chip.rpc.AttributeData.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=194,
  serialized_end=393,
)


_ATTRIBUTEWRITE = _descriptor.Descriptor(
  name='AttributeWrite',
  full_name='chip.rpc.AttributeWrite',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metadata', full_name='chip.rpc.AttributeWrite.metadata', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='chip.rpc.AttributeWrite.data', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=395,
  serialized_end=497,
)

_ATTRIBUTEMETADATA.fields_by_name['cluster'].enum_type = _CLUSTERTYPE
_ATTRIBUTEMETADATA.fields_by_name['type'].enum_type = _ATTRIBUTETYPE
_ATTRIBUTEDATA.oneofs_by_name['data'].fields.append(
  _ATTRIBUTEDATA.fields_by_name['data_bool'])
_ATTRIBUTEDATA.fields_by_name['data_bool'].containing_oneof = _ATTRIBUTEDATA.oneofs_by_name['data']
_ATTRIBUTEDATA.oneofs_by_name['data'].fields.append(
  _ATTRIBUTEDATA.fields_by_name['data_uint8'])
_ATTRIBUTEDATA.fields_by_name['data_uint8'].containing_oneof = _ATTRIBUTEDATA.oneofs_by_name['data']
_ATTRIBUTEDATA.oneofs_by_name['data'].fields.append(
  _ATTRIBUTEDATA.fields_by_name['data_uint16'])
_ATTRIBUTEDATA.fields_by_name['data_uint16'].containing_oneof = _ATTRIBUTEDATA.oneofs_by_name['data']
_ATTRIBUTEDATA.oneofs_by_name['data'].fields.append(
  _ATTRIBUTEDATA.fields_by_name['data_uint32'])
_ATTRIBUTEDATA.fields_by_name['data_uint32'].containing_oneof = _ATTRIBUTEDATA.oneofs_by_name['data']
_ATTRIBUTEDATA.oneofs_by_name['data'].fields.append(
  _ATTRIBUTEDATA.fields_by_name['data_bytes'])
_ATTRIBUTEDATA.fields_by_name['data_bytes'].containing_oneof = _ATTRIBUTEDATA.oneofs_by_name['data']
_ATTRIBUTEDATA.oneofs_by_name['data'].fields.append(
  _ATTRIBUTEDATA.fields_by_name['data_int8'])
_ATTRIBUTEDATA.fields_by_name['data_int8'].containing_oneof = _ATTRIBUTEDATA.oneofs_by_name['data']
_ATTRIBUTEDATA.oneofs_by_name['data'].fields.append(
  _ATTRIBUTEDATA.fields_by_name['data_int16'])
_ATTRIBUTEDATA.fields_by_name['data_int16'].containing_oneof = _ATTRIBUTEDATA.oneofs_by_name['data']
_ATTRIBUTEDATA.oneofs_by_name['data'].fields.append(
  _ATTRIBUTEDATA.fields_by_name['data_int32'])
_ATTRIBUTEDATA.fields_by_name['data_int32'].containing_oneof = _ATTRIBUTEDATA.oneofs_by_name['data']
_ATTRIBUTEWRITE.fields_by_name['metadata'].message_type = _ATTRIBUTEMETADATA
_ATTRIBUTEWRITE.fields_by_name['data'].message_type = _ATTRIBUTEDATA
DESCRIPTOR.message_types_by_name['AttributeMetadata'] = _ATTRIBUTEMETADATA
DESCRIPTOR.message_types_by_name['AttributeData'] = _ATTRIBUTEDATA
DESCRIPTOR.message_types_by_name['AttributeWrite'] = _ATTRIBUTEWRITE
DESCRIPTOR.enum_types_by_name['AttributeType'] = _ATTRIBUTETYPE
DESCRIPTOR.enum_types_by_name['ClusterType'] = _CLUSTERTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AttributeMetadata = _reflection.GeneratedProtocolMessageType('AttributeMetadata', (_message.Message,), {
  'DESCRIPTOR' : _ATTRIBUTEMETADATA,
  '__module__' : 'attributes_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.AttributeMetadata)
  })
_sym_db.RegisterMessage(AttributeMetadata)

AttributeData = _reflection.GeneratedProtocolMessageType('AttributeData', (_message.Message,), {
  'DESCRIPTOR' : _ATTRIBUTEDATA,
  '__module__' : 'attributes_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.AttributeData)
  })
_sym_db.RegisterMessage(AttributeData)

AttributeWrite = _reflection.GeneratedProtocolMessageType('AttributeWrite', (_message.Message,), {
  'DESCRIPTOR' : _ATTRIBUTEWRITE,
  '__module__' : 'attributes_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.AttributeWrite)
  })
_sym_db.RegisterMessage(AttributeWrite)



_ATTRIBUTES = _descriptor.ServiceDescriptor(
  name='Attributes',
  full_name='chip.rpc.Attributes',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=7274,
  serialized_end=7407,
  methods=[
  _descriptor.MethodDescriptor(
    name='Write',
    full_name='chip.rpc.Attributes.Write',
    index=0,
    containing_service=None,
    input_type=_ATTRIBUTEWRITE,
    output_type=common__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Read',
    full_name='chip.rpc.Attributes.Read',
    index=1,
    containing_service=None,
    input_type=_ATTRIBUTEMETADATA,
    output_type=_ATTRIBUTEDATA,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_ATTRIBUTES)

DESCRIPTOR.services_by_name['Attributes'] = _ATTRIBUTES

# @@protoc_insertion_point(module_scope)



"""DC2200 LED driver from Thorlabs.

SCPI programming manual:
https://www.thorlabs.com/_sd.cfm?fileName=MTN005097-D03.pdf&partNumber=DC2200

Product:
https://www.thorlabs.com/thorproduct.cfm?partnumber=DC2200
"""
from typing import Tuple

from gazoo_device import custom_types
from gazoo_device import decorators
from gazoo_device import detect_criteria
from gazoo_device import gdm_logger
from gazoo_device.base_classes import auxiliary_device
from gazoo_device.capabilities import led_driver_default
from gazoo_device.utility import usb_utils
import immutabledict


logger = gdm_logger.get_logger()


class DC2200(auxiliary_device.AuxiliaryDevice):
  """DC2200 LED driver controller."""

  COMMUNICATION_TYPE = "UsbComms"
  DETECT_MATCH_CRITERIA = immutabledict.immutabledict({
      detect_criteria.UsbQuery.VENDOR_PRODUCT_ID: "1313:80c8",
      detect_criteria.UsbQuery.PRODUCT_NAME: "DC2200",
  })
  DEVICE_TYPE = "dc2200"
  _OWNER_EMAIL = "gdm-authors@google.com"
  _MODEL = "dc2200"

  @decorators.LogDecorator(logger)
  def get_console_configuration(self) -> None:
    """Returns None. Console is not supported because there's no Switchboard."""
    del self  # Unused because console is not supported.
    return None

  def get_detection_info(
      self
  ) -> Tuple[custom_types.PersistentConfigsDict,
             custom_types.OptionalConfigsDict]:
    """Gets the persistent and optional attributes of a device during setup.

    Returns:
      Dictionaries of persistent attributes, and optional attributes.
    """
    persistent_dict = self.props["persistent_identifiers"]
    persistent_dict["serial_number"] = persistent_dict["console_port_name"]
    persistent_dict["model"] = self._MODEL
    optional_dict = {}
    return persistent_dict, optional_dict

  @classmethod
  def is_connected(
      cls, device_config: custom_types.ManagerDeviceConfigDict) -> bool:
    """Determines if the device is connected (reachable).

    Args:
      device_config: Dictionary containing "persistent" properties.

    Returns:
      True if the device is connected, False otherwise.
    """
    device = usb_utils.get_usb_device_from_serial_number(
        device_config["persistent"]["console_port_name"])
    return device is not None

  @decorators.CapabilityDecorator(led_driver_default.LedDriverDefault)
  def led_driver(self) -> led_driver_default.LedDriverDefault:
    """LED driver capability for DC2200."""
    return self.lazy_init(
        led_driver_default.LedDriverDefault,
        device_name=self.name,
        serial_number=self.serial_number)
